@@ -26,9 +26,9 @@ Core classes
 
 import hivemind.settings as settings
 import time
-from PySide.QtGui import QApplication, QFontMetricsF, QPalette, QTextDocument, \
+from PyQt4.QtGui import QApplication, QFontMetricsF, QPalette, QTextDocument, \
  QAbstractTextDocumentLayout
-from PySide.QtCore import QSizeF
+from PyQt4.QtCore import QSizeF
 from hivemind.attribute import Observable, readable, writable
 
 
@@ -542,7 +542,7 @@ class Node(Element, Observable):
                 maxNotifyLevel = max(maxNotifyLevel, Node.__LOCAL_CHANGE_LEVEL)
             elif attrName in Node.__NONEVENT_ATTRIBUTES:
                 self._setField(attrName, attrValue)
-            else: print 'Unknown attribute name: ' + attrName
+            else: raise Exception('Unknown attribute name: ' + attrName)
         # emit only one signal with max level of change
         if maxNotifyLevel != -1:
             self.notifyObservers(self, type = Node.__SIGNALS[maxNotifyLevel])
@@ -612,7 +612,7 @@ class Node(Element, Observable):
         Set node content as simple text
         '''
         self.__isHtmlContent = False
-        self.__richcontent.setPlainText(text)
+        self.__richcontent.setPlainText(text if text else '')
         self._updateTextWidth()
 
     def _setHtml(self, html):
@@ -620,7 +620,7 @@ class Node(Element, Observable):
         Set node content as formated html
         '''
         self.__isHtmlContent = True
-        self.__richcontent.setHtml(html)
+        self.__richcontent.setHtml(html if html else '')
         self._updateTextWidth()
 
     def _setPropagateNodeFormatting(self, propagateFormatting):
@@ -810,7 +810,8 @@ class Node(Element, Observable):
         
         @rtype: QSizeF
         '''
-        return self.__richcontent.size()
+        size = self.__richcontent.size()
+        return QSizeF(self.__richcontent.idealWidth(), size.height())
 
     @property
     def labelSize(self):
