@@ -24,14 +24,14 @@
 Dialogs for Desktop platform
 '''
 
-from PySide.QtGui import QToolBar, QIcon, QTabWidget, QGridLayout, \
+from PyQt4.QtGui import QToolBar, QIcon, QTabWidget, QGridLayout, \
 QHBoxLayout, QPalette, QTextBrowser, QFrame, QLabel, QDialogButtonBox, \
-QVBoxLayout, QGroupBox, QWidget
-from PySide.QtCore import SIGNAL, QSize, Qt
+QVBoxLayout, QGroupBox, QWidget, QPushButton, QKeySequence, QScrollArea
+from PyQt4.QtCore import SIGNAL, QSize, Qt, QObject
 from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
 AboutDialog, EditEdgeDialog, SettingsDialog, NetworkConnectionDialog, \
-AffiliationDialog, NetworkServiceDialog, NetworkClientDialog
-from hivemind.gui_widgets import SettingsCreator
+AffiliationDialog, NetworkSettingsDialog, NetworkClientDialog
+from hivemind.gui_widgets import SettingsCreator, FindToolBar
 from hivemind.attribute import adaptCallable
 import hivemind.__version__ as __version__
 
@@ -55,6 +55,30 @@ class DesktopEditNodeDialog(EditNodeDialog):
     def _initComponents(self):
         '''Form initialization'''
         #pylint: disable = W0201
+        self.__tabWidget = QTabWidget(self)
+        self.__textIndex = self.__tabWidget.addTab(self._editor, self.tr('Text'))
+        self.__htmlIndex = self.__tabWidget.addTab(self._htmlEditor, 'Html')
+        mainLayout = QVBoxLayout()
+        mainLayout.addWidget(self._createToolbar())
+        mainLayout.addWidget(self.__tabWidget)
+        lastLineLayout = QHBoxLayout()
+        lastLineLayout.addWidget(self._propagateFormatting)
+        lastLineLayout.addStretch()
+        lastLineLayout.addWidget(self._buttonBox)
+        mainLayout.addLayout(lastLineLayout)
+        QWidget.setTabOrder(self._editor, self.__tabWidget)
+        QObject.connect(self._textHtmlAction, SIGNAL('toggled(bool)'),
+                adaptCallable(self._enableSourceTab))
+        QObject.connect(self.__tabWidget, SIGNAL('currentChanged(int)'),
+                adaptCallable(self._changedMode))
+        self.setLayout(mainLayout)
+        self._editor.setFocus()
+
+    def _createToolbar(self):
+        '''
+        Create and return dialog toolbar
+        @rtype QToolBar
+        '''
         toolbar = QToolBar(self)
         toolbar.addWidget(self._comboFont)
         toolbar.addWidget(self._comboSize)
@@ -79,24 +103,7 @@ class DesktopEditNodeDialog(EditNodeDialog):
         self._clearFormatAction.setToolTip(self.tr('Clear format'))
         self._clearFormatAction.setIcon(QIcon.fromTheme('edit-clear'))
         toolbar.addAction(self._clearFormatAction)
-        self.__tabWidget = QTabWidget(self)
-        self.__textIndex = self.__tabWidget.addTab(self._editor, self.tr('Text'))
-        self.__htmlIndex = self.__tabWidget.addTab(self._htmlEditor, 'Html')
-        mainLayout = QVBoxLayout()
-        mainLayout.addWidget(toolbar)
-        mainLayout.addWidget(self.__tabWidget)
-        lastLineLayout = QHBoxLayout()
-        lastLineLayout.addWidget(self._propagateFormatting)
-        lastLineLayout.addStretch()
-        lastLineLayout.addWidget(self._buttonBox)
-        mainLayout.addLayout(lastLineLayout)
-        QWidget.setTabOrder(self._editor, self.__tabWidget)
-        self._textHtmlAction.connect(SIGNAL('toggled(bool)'),
-                adaptCallable(self._enableSourceTab))
-        self.__tabWidget.connect(SIGNAL('currentChanged(int)'),
-                adaptCallable(self._changedMode))
-        self.setLayout(mainLayout)
-        self._editor.setFocus()
+        return toolbar
 
     def _changedMode(self, isSourceMode):
         '''
@@ -243,13 +250,14 @@ class DesktopAboutDialog(AboutDialog):
         authors = QTextBrowser()
         authors.setFrameStyle(QFrame.NoFrame)
         authors.setOpenExternalLinks(True)
-        bugReport = self.tr('Please use %s to report bugs') % (self._linkTemplate %
-                (self._bugLink, self._siteLink))
+        bugReport = unicode(self.tr('Please use %s to report bugs')) % \
+                (self._linkTemplate % (self._bugLink, self._siteLink))
         authors.setHtml('%s<br>' % (bugReport) +
                 self._generateInfo(self._authorInfo, 'AndreyVasilev') +
+                self._generateInfo(self._authorInfo, 'IlyaParamonov') +
                 self._generateInfo(self._authorInfo, 'OlegKandaurov') +
                 self._generateInfo(self._authorInfo, 'AlexanderKulikov') +
-                self._generateInfo(self._authorInfo, 'IlyaParamonov') +
+                self._generateInfo(self._authorInfo, 'EldarMamedov') +
                 self._generateInfo(self._authorInfo, 'AndreyGolovchenko'))
         self._tabWidget.addTab(authors, self.tr('A&uthors'))
         thanks = QTextBrowser()
@@ -280,9 +288,10 @@ class DesktopAboutDialog(AboutDialog):
         mainLayout.addWidget(self._tabWidget)
         mainLayout.addWidget(close)
         self.setLayout(mainLayout)
-        close.connect(SIGNAL('rejected()'), self.close)
+        QObject.connect(close, SIGNAL('rejected()'), self.close)
 
-    def _generateInfo(self, info, name):
+    @classmethod
+    def _generateInfo(cls, info, name):
         '''
         Generate formatted information about person
         @type info: dict
@@ -323,7 +332,6 @@ class DesktopEditEdgeDialog(EditEdgeDialog):
         mainLayout.addWidget(self._buttonBox, 5, 0, 1, 2)
         self.setLayout(mainLayout)
 
-
 class DesktopSettingsDialog(SettingsDialog):
     '''
     Settings dialog for Desktop platform
@@ -349,11 +357,18 @@ class DesktopSettingsDialog(SettingsDialog):
         self._autosave = SettingsCreator(['autosaveEnabled', 'autosaveInterval'])
         autosaveTab = QWidget()
         autosaveTab.setLayout(self._autosave.getLayout())
+        shortcutsTab = QWidget()
+        self._shortcuts = SettingsCreator(['shortcuts'])
+        shortcutsTab.setLayout(self._shortcuts.getLayout())
+        shortcutsScrollArea = QScrollArea()
+        shortcutsScrollArea.setWidgetResizable(True)
+        shortcutsScrollArea.setWidget(shortcutsTab)
         tabWidget = QTabWidget()
         tabWidget.addTab(generalTab, self.tr('General'))
         tabWidget.addTab(nodeTab, self.tr('Node'))
         tabWidget.addTab(edgeTab, self.tr('Edge && label'))
         tabWidget.addTab(autosaveTab, self.tr('Autosave'))
+        tabWidget.addTab(shortcutsScrollArea, self.tr('Shortcuts'))
         tabWidget.setUsesScrollButtons(False)
         mainLayout = QVBoxLayout()
         mainLayout.addWidget(tabWidget)
@@ -368,6 +383,7 @@ class DesktopSettingsDialog(SettingsDialog):
         self._newSettings.append(self._node.retrieve())
         self._newSettings.append(self._edge.retrieve())
         self._newSettings.append(self._autosave.retrieve())
+        self._newSettings.append(self._shortcuts.retrieve())
         SettingsDialog._accept(self)
 
 
@@ -380,14 +396,14 @@ class DesktopAffiliationDialog(AffiliationDialog):
         AffiliationDialog.__init__(self, model, parent)
 
 
-class DesktopNetworkServiceDialog(NetworkServiceDialog):
+class DesktopNetworkSettingsDialog(NetworkSettingsDialog):
     '''
-    Service dialog for Desktop platfrom
+    Network settings dialog for Desktop platfrom
     @author: Oleg Kandaurov
     '''
 
     def __init__(self, parent = None):
-        NetworkServiceDialog.__init__(self, parent)
+        NetworkSettingsDialog.__init__(self, parent)
 
 
 class DesktopNetworkClientDialog(NetworkClientDialog):
@@ -408,3 +424,55 @@ class DesktopNetworkConnectionDialog(NetworkConnectionDialog):
 
     def __init__(self, parent = None):
         NetworkConnectionDialog.__init__(self, parent)
+
+
+class DesktopFindToolBar(FindToolBar):
+    '''
+    Find tool bar for Desktop platform
+    '''
+    def __init__(self, controller, parent = None):
+        '''
+        @type controller: MainWindowController
+        @type parent: QWidget
+        '''
+        FindToolBar.__init__(self, controller, parent)
+        self.setAllowedAreas(Qt.TopToolBarArea | Qt.BottomToolBarArea)
+        self._createComponents()
+        self._initComponents()
+
+    def _createComponents(self):
+        '''
+        Create components
+        '''
+        #pylint: disable = W0201
+        self.__prevButton = QPushButton(self.tr('&Previous'))
+        self.__nextButton = QPushButton(self.tr('&Next'))
+        self.__closeButton = QPushButton(self.tr('&Close'))
+        self.__closeButton.setIcon(QIcon.fromTheme('window-close'))
+        self.__prevButton.setIcon(QIcon.fromTheme('go-previous'))
+        self.__nextButton.setIcon(QIcon.fromTheme('go-next'))
+        self.__prevButton.setShortcut(QKeySequence.FindPrevious)
+        self.__nextButton.setShortcut(QKeySequence.FindNext)
+        QObject.connect(self.__nextButton, SIGNAL('clicked()'), self._finder.nextFoundNode)
+        QObject.connect(self.__prevButton, SIGNAL('clicked()'), self._finder.prevFoundNode)
+        QObject.connect(self.__closeButton, SIGNAL('clicked()'), self.close)
+
+    def _initComponents(self):
+        '''
+        Init components
+        '''
+        FindToolBar._createComponents(self)
+        self._findLineEdit.setFixedWidth(self._findLineEdit.sizeHint().width())
+        mainLayout = QHBoxLayout()
+        mainLayout.addWidget(self._findLabel)
+        mainLayout.addWidget(self._findLineEdit)
+        mainLayout.addWidget(self.__prevButton)
+        mainLayout.addWidget(self.__nextButton)
+        mainLayout.addWidget(self._highlightCheckBox)
+        mainLayout.addWidget(self._caseSensitiveCheckBox)
+        mainLayout.addStretch()
+        mainLayout.addWidget(self._statusLabel)
+        mainLayout.addStretch()
+        mainLayout.addWidget(self.__closeButton)
+        self._findWidget.setLayout(mainLayout)
+        self.addWidget(self._findWidget)
