@@ -24,9 +24,11 @@
 Main window class for Desktop platform
 '''
 
-from PySide.QtCore import SIGNAL, Qt
-from PySide.QtGui import QMenu, QCursor, QMainWindow
+import hivemind.resource as resource
+from PyQt4.QtCore import SIGNAL, Qt
+from PyQt4.QtGui import QMenu, QCursor, QMainWindow, QIcon
 from hivemind.main_window import MainView, AbstractMainWindow
+from hivemind.attribute import readable
 
 
 class DesktopMainView(MainView):
@@ -57,8 +59,8 @@ class DesktopMainView(MainView):
             item = self.scene().itemAt(self.mapToScene(event.pos()))
             parent, childLocation = self.scene().parentAt(self.mapToScene(event.pos()))
             if parent is not None:
-                self.emit(SIGNAL('itemClicked'), event.button() == Qt.RightButton, item, parent,
-                        childLocation)
+                self.emit(SIGNAL('itemClicked'), event.button() == Qt.RightButton, item,
+                        parent, childLocation)
             if item is None:
                 self.emit(SIGNAL('mapClicked'))
 
@@ -69,12 +71,16 @@ class DesktopMainWindow(AbstractMainWindow):
 
     @author: Alexander Kulikov
     '''
+
+    readable('mindMapView')
+
     def __init__(self, actionBag, mindMapScene):
         '''Create new window'''
         AbstractMainWindow.__init__(self, actionBag, mindMapScene)
-        self._mindMapView = DesktopMainView(mindMapScene, actionBag)
-        self.setCentralWidget(self._mindMapView)
-        self._createContextMenu()
+        self.setWindowIcon(QIcon(resource.getScalableImage('hivemind')))
+        self.__mindMapView = DesktopMainView(mindMapScene, actionBag)
+        self.setCentralWidget(self.__mindMapView)
+        self.__contextMenu = self._createContextMenu()
         self._createMainMenu()
         self._createToolBars()
 
@@ -86,30 +92,35 @@ class DesktopMainWindow(AbstractMainWindow):
 
     def _createContextMenu(self):
         '''Create context menu'''
-        self.__menu = QMenu(self)
-        self.__menu.addAction(self._actionBag.addNodeAction)
-        self.__menu.addAction(self._actionBag.addSiblingNodeAboveAction)
-        self.__menu.addAction(self._actionBag.addSiblingNodeBelowAction)
-        self.__menu.addSeparator()
-        self.__menu.addAction(self._actionBag.foldNodeAction)
-        self.__menu.addAction(self._actionBag.enterTransferModeAction)
-        self.__menu.addAction(self._actionBag.removeNodeAction)
-        self.__menu.addSeparator()
-        self.__menu.addAction(self._actionBag.editNodeAction)
-        self.__menu.addAction(self._actionBag.editEdgeAction)
-        self.__menu.addAction(self._actionBag.editLabelAction)
-        self.__menu.addAction(self._actionBag.editNodeIconsAction)
+        menu = QMenu(self)
+        menu.addAction(self._actionBag.addNodeAction)
+        menu.addAction(self._actionBag.addSiblingNodeAboveAction)
+        menu.addAction(self._actionBag.addSiblingNodeBelowAction)
+        menu.addSeparator()
+        menu.addAction(self._actionBag.foldNodeAction)
+        menu.addAction(self._actionBag.enterTransferModeAction)
+        menu.addAction(self._actionBag.removeNodeAction)
+        menu.addSeparator()
+        menu.addAction(self._actionBag.editNodeAction)
+        menu.addAction(self._actionBag.editEdgeAction)
+        menu.addAction(self._actionBag.editLabelAction)
+        menu.addAction(self._actionBag.editNodeIconsAction)
+        menu.addAction(self._actionBag.editHyperlinkAction)
+        menu.addAction(self._actionBag.openHyperlinkAction)
+        return menu
 
     def _addEllipsesToActionText(self):
         '''Add ellipses to the text of the actions displaying dialogs'''
         actionsWithEllipses = [
                 self._actionBag.fileOpenAction, self._actionBag.fileSaveAsAction,
-                self._actionBag.settingsAction, self._actionBag.aboutAction,
+                self._actionBag.settingsAction, self._actionBag.exportToImageAction,
                 self._actionBag.addNodeAction, self._actionBag.addSiblingNodeBelowAction,
                 self._actionBag.addSiblingNodeAboveAction, self._actionBag.editNodeAction,
                 self._actionBag.editNodeIconsAction, self._actionBag.editLabelAction,
                 self._actionBag.editEdgeAction, self._actionBag.editAffiliationAction,
-                self._actionBag.startServiceAction, self._actionBag.startClientAction]
+                self._actionBag.startClientAction, self._actionBag.networkSettingsAction,
+                self._actionBag.printAction, self._actionBag.findAction,
+                self._actionBag.editHyperlinkAction, self._actionBag.startServiceAction]
         for action in actionsWithEllipses:
             action.setText(action.text() + '...')
 
@@ -122,11 +133,19 @@ class DesktopMainWindow(AbstractMainWindow):
         menu.addAction(self._actionBag.fileSaveAction)
         menu.addAction(self._actionBag.fileSaveAsAction)
         menu.addSeparator()
+        menu.addAction(self._actionBag.exportToImageAction)
+        menu.addAction(self._actionBag.printAction)
+        menu.addSeparator()
+        for action in self._actionBag.recentFileActions:
+            menu.addAction(action)
+        menu.addSeparator()
         menu.addAction(self._actionBag.exitAction)
         menu = self.menuBar().addMenu(self.tr('&Edit'))
         menu.addAction(self._actionBag.undoAction)
         menu.addAction(self._actionBag.redoAction)
         menu.addSeparator()
+        menu.addAction(self._actionBag.findAction)
+        menu.addSeparator()
         menu.addAction(self._actionBag.settingsAction)
         menu = self.menuBar().addMenu(self.tr('&Network'))
         menu.addAction(self._actionBag.startServiceAction)
@@ -134,6 +153,12 @@ class DesktopMainWindow(AbstractMainWindow):
         menu.addAction(self._actionBag.editAffiliationAction)
         menu.addAction(self._actionBag.stopProtocolAction)
         menu.addAction(self._actionBag.disconnectXMPPAction)
+        menu.addSeparator()
+        menu.addAction(self._actionBag.networkSettingsAction)
+#        menu.addAction(self._actionBag.networkHelpAction)
+        menu = self.menuBar().addMenu(self.tr('SmartConference'))
+        menu.addAction(self._actionBag.startSmartConferenceRobot)
+        menu.addAction(self._actionBag.stopSmartConferenceRobot)
         menu = self.menuBar().addMenu(self.tr('&Help'))
         menu.addAction(self._actionBag.aboutAction)
 
@@ -152,8 +177,8 @@ class DesktopMainWindow(AbstractMainWindow):
         toolbar.addAction(self._actionBag.zoomInAction)
         toolbar = self.addToolBar(self.tr('Connection status'))
         toolbar.addAction(self._actionBag.connectionStatusAction)
-        toolbar.addAction(self._actionBag.subscribeStatusAction)
 
     def showMenu(self, rightButton, item):
         '''Show the node menu on right button click'''
-        self.__menu.exec_(QCursor.pos())
+        if rightButton:
+            self.__contextMenu.exec_(QCursor.pos())
