@@ -27,20 +27,21 @@ Abstract dialog classes
 
 import hivemind.settings as settings
 import hivemind.resource as resource
-from PySide.QtGui import QDialog, QMessageBox, QDialogButtonBox, QLabel, QLineEdit, \
+from PyQt4.QtGui import QDialog, QMessageBox, QDialogButtonBox, QLabel, QLineEdit, \
 QVBoxLayout, QFormLayout, QAbstractItemView, QTableView, QHeaderView, QHBoxLayout, \
-QRadioButton, QButtonGroup, QComboBox, QPushButton, QFontComboBox, QAction, QFontDatabase, \
-QColorDialog, QTextCharFormat, QFont, QPalette, QBrush, QTextCursor, QGridLayout, QCheckBox, \
-QGraphicsView, QGroupBox
-from PySide.QtCore import SIGNAL, QRegExp, Qt, QSize, SLOT, QCoreApplication
+QComboBox, QPushButton, QFontComboBox, QAction, QFontDatabase, QColorDialog, QTextCharFormat, \
+QFont, QPalette, QBrush, QTextCursor, QGridLayout, QCheckBox, QGraphicsView, \
+QTabWidget, QWidget, QSpinBox
+from PyQt4.QtCore import SIGNAL, QRegExp, Qt, QSize, SLOT, QCoreApplication, QObject
 from hivemind.attribute import readable, adaptCallable
-from hivemind.gui_widgets import SearchingTrMixin, SpecialTextEdit, \
-ScalingWidget, IconList, IconTable, LineEditBinder, CheckBoxBinder, ColorLabel
+from hivemind.gui_widgets import SearchingTrMixin, SpecialTextEdit, SettingsCreator, \
+ScalingWidget, IconList, IconTable, LineEditBinder, ColorLabel
 from hivemind.network.network_models import AffiliationDelegate
 from twisted.words.protocols.jabber import jid
 
 '''Regexp for jid validation'''
 VALID_JID = '[A-Za-z0-9._%+-]+@(?:[A-Za-z0-9-]+\\.)+[A-Za-z]{2,6}'
+VALID_HOST = '([A-Za-z0-9-]+\\.)+[A-Za-z]{2,6}'
 
 
 def _showJIDErrorDialog(parent):
@@ -69,8 +70,8 @@ class CommonDialog(SearchingTrMixin, QDialog):
         self._buttonBox = QDialogButtonBox()
         self._okButton = self._buttonBox.addButton(QDialogButtonBox.Ok)
         self._cancelButton = self._buttonBox.addButton(QDialogButtonBox.Cancel)
-        self._buttonBox.connect(SIGNAL('accepted()'), self._accept)
-        self._buttonBox.connect(SIGNAL('rejected()'), self.close)
+        QObject.connect(self._buttonBox, SIGNAL('accepted()'), self._accept)
+        QObject.connect(self._buttonBox, SIGNAL('rejected()'), self.close)
 
     def _accept(self):
         '''
@@ -104,15 +105,15 @@ class EditNodeDialog(CommonDialog):
         #pylint: disable = W0201
         # init edit widget
         self._editor = SpecialTextEdit()
-        self.connect(self._editor, SIGNAL('accept'), self._accept)
+        QObject.connect(self._editor, SIGNAL('accept'), self._accept)
         # init html widget
         self._htmlEditor = SpecialTextEdit()
         # create font toolbar
         self._comboFont = QFontComboBox()
         self._comboSize = QComboBox()
         self._comboSize.setEditable(True)
-        self._comboFont.connect(SIGNAL('currentFontChanged(QFont)'), self._buildFont)
-        self._comboSize.connect(SIGNAL('currentIndexChanged(int)'), self._buildFont)
+        QObject.connect(self._comboFont, SIGNAL('currentFontChanged(QFont)'), self._buildFont)
+        QObject.connect(self._comboSize, SIGNAL('currentIndexChanged(int)'), self._buildFont)
         # text format actions
         self._boldAction = QAction(self)
         self._boldAction.setCheckable(True)
@@ -132,16 +133,19 @@ class EditNodeDialog(CommonDialog):
         self._propagateFormatting = QCheckBox(self.tr('Propagate formatting to child nodes'))
         # clear format action
         self._clearFormatAction = QAction(self)
+        # edit hyperlink dialog action
+        self._editHyperlinkAction = QAction(self)
         # actions signals connection
-        self._boldAction.connect(SIGNAL('triggered(bool)'), self._setBold)
-        self._italicAction.connect(SIGNAL('triggered(bool)'), self._setItalic)
-        self._underlineAction.connect(SIGNAL('triggered(bool)'), self._setUnderline)
-        self._colorAction.connect(SIGNAL('triggered()'), self._textColor)
-        self._backgroundAction.connect(SIGNAL('triggered()'), self._textBackground)
-        self._textHtmlAction.connect(SIGNAL('triggered()'), self._textHtml)
-        self._clearFormatAction.connect(SIGNAL('triggered()'), self._clearFormat)
-        self._textFontAction.connect(SIGNAL('triggered()'), self._textFont)
-        self._editor.connect(SIGNAL('currentCharFormatChanged(QTextCharFormat)'),
+        QObject.connect(self._boldAction, SIGNAL('triggered(bool)'), self._setBold)
+        QObject.connect(self._italicAction, SIGNAL('triggered(bool)'), self._setItalic)
+        QObject.connect(self._underlineAction, SIGNAL('triggered(bool)'), self._setUnderline)
+        QObject.connect(self._colorAction, SIGNAL('triggered()'), self._textColor)
+        QObject.connect(self._backgroundAction, SIGNAL('triggered()'), self._textBackground)
+        QObject.connect(self._textHtmlAction, SIGNAL('triggered()'), self._textHtml)
+        QObject.connect(self._clearFormatAction, SIGNAL('triggered()'), self._clearFormat)
+        QObject.connect(self._textFontAction, SIGNAL('triggered()'), self._textFont)
+        QObject.connect(self._editHyperlinkAction, SIGNAL('triggered()'), self._editHyperlink)
+        QObject.connect(self._editor, SIGNAL('currentCharFormatChanged(QTextCharFormat)'),
                              self._currentCharFormatChanged)
 
     def _currentCharFormatChanged(self, textFormat):
@@ -189,6 +193,7 @@ class EditNodeDialog(CommonDialog):
         storeAttrs['color'] = self.__node.color
         storeAttrs['backgroundColor'] = self.__node.backgroundColor
         storeAttrs['propagateNodeFormatting'] = self._propagateFormatting.isChecked()
+        storeAttrs['link'] = self.__node.link
         self.__newAttributes = storeAttrs
         QDialog.accept(self)
 
@@ -336,6 +341,14 @@ class EditNodeDialog(CommonDialog):
         cursor.mergeCharFormat(textFormat)
         self._editor.mergeCurrentCharFormat(textFormat)
 
+    def _editHyperlink(self):
+        '''
+        Exec edit hyperlink dialog
+        '''
+        editHyperlinkDialog = EditHyperlinkDialog(self.__node, '', self)
+        if editHyperlinkDialog.exec_() == QDialog.Accepted:
+            self.__node.setAttributes(**(editHyperlinkDialog.attributes()))
+
 
 class FontDialog(CommonDialog):
     '''
@@ -381,8 +394,8 @@ class FontDialog(CommonDialog):
         layout.addWidget(scalingWidget, 1, 0, 1, 4)
         layout.addWidget(self._buttonBox, 2, 0, 1, 4)
         self.setLayout(layout)
-        self.__comboFont.connect(SIGNAL('currentFontChanged(QFont)'), self._setLabelFont)
-        self.__comboSize.connect(SIGNAL('currentIndexChanged(int)'), self._setLabelSize)
+        QObject.connect(self.__comboFont, SIGNAL('currentFontChanged(QFont)'), self._setLabelFont)
+        QObject.connect(self.__comboSize, SIGNAL('currentIndexChanged(int)'), self._setLabelSize)
         self._setLabelFormat()
 
     def _accept(self):
@@ -427,8 +440,8 @@ class EditLabelDialog(CommonDialog):
         CommonDialog.__init__(self, parent)
         self.setWindowTitle(self.tr('Edit Label'))
         self.__node = node.clone()
-        self.__color = None
-        self.__font = None
+        self.__color = self.__node.labelColor
+        self.__font = self.__node.labelFont
         self.__newAttributes = {}
         self._createComponents()
         self._initEditor()
@@ -445,40 +458,39 @@ class EditLabelDialog(CommonDialog):
         self._underlineAction.setCheckable(True)
         self._comboFont = QFontComboBox()
         self._comboSize = QComboBox()
-        self.connect(self._editor, SIGNAL('accept'), self._accept)
-        self._comboFont.connect(SIGNAL('currentFontChanged(QFont)'), self._setFont)
-        self._comboSize.connect(SIGNAL('currentIndexChanged(int)'), self._setSize)
+        QObject.connect(self._editor, SIGNAL('accept'), self._accept)
+        QObject.connect(self._comboFont, SIGNAL('currentFontChanged(QFont)'), self._setFont)
+        QObject.connect(self._comboSize, SIGNAL('currentIndexChanged(int)'), self._setSize)
         self._parentColor = QCheckBox(self.tr("Parent's color"))
         self._colorButton = QPushButton(self.tr('Choose color'))
         self._parentFont = QCheckBox(self.tr("Parent's font"))
         self._propagateFormatting = QCheckBox(self.tr('Propagate formatting to child nodes'))
-        self._boldAction.connect(SIGNAL('triggered(bool)'), self._setBold)
-        self._italicAction.connect(SIGNAL('triggered(bool)'), self._setItalic)
-        self._underlineAction.connect(SIGNAL('triggered(bool)'), self._setUnderline)
-        self._colorButton.connect(SIGNAL('clicked()'), self._textColor)
-        self._parentColor.connect(SIGNAL('toggled(bool)'), self._colorButton,
+        QObject.connect(self._boldAction, SIGNAL('triggered(bool)'), self._setBold)
+        QObject.connect(self._italicAction, SIGNAL('triggered(bool)'), self._setItalic)
+        QObject.connect(self._underlineAction, SIGNAL('triggered(bool)'), self._setUnderline)
+        QObject.connect(self._colorButton, SIGNAL('clicked()'), self._textColor)
+        QObject.connect(self._parentColor, SIGNAL('toggled(bool)'), self._colorButton,
                                   SLOT('setDisabled(bool)'))
-        self._parentColor.connect(SIGNAL('toggled(bool)'), self._updateEditor)
-        self._parentFont.connect(SIGNAL('toggled(bool)'), self._comboFont,
+        QObject.connect(self._parentColor, SIGNAL('toggled(bool)'), self._updateEditor)
+        QObject.connect(self._parentFont, SIGNAL('toggled(bool)'), self._comboFont,
                                  SLOT('setDisabled(bool)'))
-        self._parentFont.connect(SIGNAL('toggled(bool)'), self._comboSize,
+        QObject.connect(self._parentFont, SIGNAL('toggled(bool)'), self._comboSize,
                                  SLOT('setDisabled(bool)'))
-        self._parentFont.connect(SIGNAL('toggled(bool)'), self._boldAction,
+        QObject.connect(self._parentFont, SIGNAL('toggled(bool)'), self._boldAction,
                                  SLOT('setDisabled(bool)'))
-        self._parentFont.connect(SIGNAL('toggled(bool)'), self._italicAction,
+        QObject.connect(self._parentFont, SIGNAL('toggled(bool)'), self._italicAction,
                                  SLOT('setDisabled(bool)'))
-        self._parentFont.connect(SIGNAL('toggled(bool)'), self._underlineAction,
+        QObject.connect(self._parentFont, SIGNAL('toggled(bool)'), self._underlineAction,
                                  SLOT('setDisabled(bool)'))
-        self._parentFont.connect(SIGNAL('toggled(bool)'), self._updateEditor)
+        QObject.connect(self._parentFont, SIGNAL('toggled(bool)'), self._updateEditor)
 
     def _initEditor(self):
         '''Initialize editor'''
-        self._editor.setPlainText(self.__node.labelText)
+        if self.__node.labelText:
+            self._editor.setPlainText(self.__node.labelText)
         self._editor.document().setDefaultFont(settings.get('defaultLabelFont'))
-        self.__font = self.__node.labelFont
         self._setEditorFont(self.__node.labelFont)
         self._parentFont.setChecked(self.__node.labelFont is None)
-        self.__color = self.__node.labelColor
         self._setEditorColor(self.__node.labelColor)
         self._parentColor.setChecked(self.__node.labelColor is None)
         self._comboSize.setEditable(True)
@@ -543,6 +555,10 @@ class EditLabelDialog(CommonDialog):
         font = self._comboFont.currentFont()
         if index != -1:
             font.setPointSize(int(self._comboSize.itemText(index)))
+        if not self.__font: self.__font = self.__node.effectiveLabelFont
+        font.setItalic(self.__font.italic())
+        font.setUnderline(self.__font.underline())
+        font.setBold(self.__font.bold())
         self._setEditorFont(font)
         self._updateButtonsState(font)
         self.__font = font
@@ -566,7 +582,7 @@ class EditLabelDialog(CommonDialog):
         Update text editor if Underline button has triggered
         '''
         self.__font.setUnderline(checked)
-        self._setEditorColor(self.__font)
+        self._setEditorFont(self.__font)
 
     def _textColor(self):
         '''Choose font color'''
@@ -631,7 +647,7 @@ class EditLabelDialog(CommonDialog):
         modified = {}
         modified['labelColor'] = self.__node.labelColor
         modified['labelFont'] = self.__node.labelFont
-        modified['labelText'] = self._editor.toPlainText().replace('\n', ' ')
+        modified['labelText'] = unicode(self._editor.toPlainText()).replace('\n', ' ')
         modified['propagateLabelFormatting'] = self._propagateFormatting.isChecked()
         self.__newAttributes = modified
         QDialog.accept(self)
@@ -676,8 +692,8 @@ class IconDialog(CommonDialog):
         self._iconList = QGraphicsView(self._listScene, self)
         self._iconList.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self._iconList.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        self._iconTable.connect(SIGNAL('iconClicked'), \
-                                 adaptCallable(self._listScene.addIcon))
+        QObject.connect(self._iconTable, SIGNAL('iconClicked'),
+                adaptCallable(self._listScene.addIcon))
 
     def _accept(self):
         '''
@@ -723,19 +739,21 @@ class AboutDialog(CommonDialog):
         self._appDescription = self.tr('Collaborative mind map editor')
         self._info = QLabel(self._appDescription)
         self._authorInfo = {'AndreyGolovchenko': {'name': self.tr('Andrey Golovchenko'),
-                'site': 'mailto:golovchenkoaa@gmail.com', 'role': self.tr('core developer')},
+                'site': 'mailto:golovchenkoaa@gmail.com', 'role': self.tr('developer')},
                 'AndreyVasilev': {'name': self.tr('Andrey Vasilev'),
-                'site': 'mailto:vamonster@gmail.com', 'role': self.tr('developer/designer')},
+                'site': 'mailto:vamonster@gmail.com', 'role': self.tr('project leader/designer')},
                 'AlexanderKulikov': {'name': self.tr('Alexander Kulikov'),
                 'site': 'mailto:asgard1988@yandex.ru', 'role': self.tr('developer')},
                 'OlegKandaurov': {'name': self.tr('Oleg Kandaurov'),
                 'site': 'mailto:kandaurov.oleg@gmail.com', 'role': self.tr('developer')},
                 'IlyaParamonov': {'name': self.tr('Ilya Paramonov'),
-                'site': 'mailto:ivparamonov@gmail.com', 'role': self.tr('technical manager')}}
+                'site': 'mailto:ivparamonov@gmail.com', 'role': self.tr('technical manager')},
+                'EldarMamedov': {'name': self.tr('Eldar Mamedov'),
+                'site': 'mailto:eldar.mamedov@list.ru', 'role': self.tr('developer')}}
         self._participantInfo = {'NicuBuculei': {'name': self.tr('Nicu Buculei'),
                 'site': 'http://nicubunu.ro', 'role': self.tr('HiveMind logo')},
                 'Mentalrey': {'name': self.tr('Mentalrey'),
-                'site': 'http://www.mentalrey.it', 'role': self.tr('Network icons')}}
+                'site': 'http://www.mentalrey.it', 'role': self.tr('Icons')}}
         self._licenseLink = 'http://www.gnu.org/licenses/gpl-3.0.html'
         self._licenseInfo = self.tr('GNU General Public License Version 3')
         self._fruct = self.tr('Open Innovations Framework Program FRUCT')
@@ -743,11 +761,11 @@ class AboutDialog(CommonDialog):
         self._bugLink = 'http://linuxlab.corp7.uniyar.ac.ru/projects/hivemind/issues/new'
         self._siteLink = 'http://linuxlab.corp7.uniyar.ac.ru/'
         self._linkTemplate = '<a href="%s">%s</a>'
-        self._copyright = '%s &copy; 2010-2011 %s' % (self.tr('Copyright'),
-                self.tr('%s developers') % self._appName)
-        self._license = self.tr('License: %s') % (self._linkTemplate % (self._licenseLink,
+        self._copyright = '%s &copy; 2010-2011 %s' % (unicode(self.tr('Copyright')),
+                unicode(self.tr('%s developers')) % self._appName)
+        self._license = unicode(self.tr('License: %s')) % (self._linkTemplate % (self._licenseLink,
                 self._licenseInfo))
-        self._development = self.tr('%s development is supported by') % self._appName
+        self._development = unicode(self.tr('%s development is supported by')) % self._appName
 
 
 class EditEdgeDialog(CommonDialog):
@@ -801,8 +819,8 @@ class EditEdgeDialog(CommonDialog):
         self._colorLabel = QLabel(self.tr('Color'))
         self._parentColor = QCheckBox(self.tr('Parent'))
         self._colorButton = QPushButton(self.tr('Choose color'))
-        self._colorButton.connect(SIGNAL('clicked()'), self._edgeColor)
-        self._parentColor.connect(SIGNAL('toggled(bool)'), self._setParentColor)
+        QObject.connect(self._colorButton, SIGNAL('clicked()'), self._edgeColor)
+        QObject.connect(self._parentColor, SIGNAL('toggled(bool)'), self._setParentColor)
         self._colorLabel = ColorLabel()
 
     def _initEditor(self):
@@ -852,12 +870,12 @@ class EditEdgeDialog(CommonDialog):
         '''
         modified = {}
         modified['edgeColor'] = self.__node.edgeColor
-        modified['edgeWidth'] = self._edgeWidthComboBox.itemData(
-                self._edgeWidthComboBox.currentIndex())
-        modified['edgeStyle'] = self._edgeStyleComboBox.itemData(
-                self._edgeStyleComboBox.currentIndex())
-        modified['style'] = self._nodeStyleComboBox.itemData(
-                self._nodeStyleComboBox.currentIndex())
+        data = self._edgeWidthComboBox.itemData(self._edgeWidthComboBox.currentIndex())
+        modified['edgeWidth'] = data.toInt()[0] if data.isNull() else None
+        data = self._edgeStyleComboBox.itemData(self._edgeStyleComboBox.currentIndex())
+        modified['edgeStyle'] = unicode(data.toString()) if data.isNull() else None
+        data = self._nodeStyleComboBox.itemData(self._nodeStyleComboBox.currentIndex())
+        modified['style'] = unicode(data.toString()) if data.isNull() else None
         self.__newAttributes = modified
         QDialog.accept(self)
 
@@ -869,6 +887,7 @@ class EditEdgeDialog(CommonDialog):
         '''
         return self.__newAttributes
 
+
 class SettingsDialog(CommonDialog):
     '''
     Abstract class for settings dialog
@@ -925,8 +944,8 @@ class SideSelectionDialog(CommonDialog):
         self.__label = QLabel(self.tr('Select node position'))
         self.__left = QPushButton(self.tr('Left'))
         self.__right = QPushButton(self.tr('Right'))
-        self.connect(self.__left, SIGNAL('clicked()'), self._accept)
-        self.connect(self.__right, SIGNAL('clicked()'), self._accept)
+        QObject.connect(self.__left, SIGNAL('clicked()'), self._accept)
+        QObject.connect(self.__right, SIGNAL('clicked()'), self._accept)
 
     def _initComponents(self):
         '''
@@ -981,7 +1000,8 @@ class AffiliationDialog(CommonDialog):
         for data, name in self.__model.mapAffiliations.iteritems():
             self.__affiliationCombo.addItem(name, data)
         self.__additionButton = QPushButton(self.tr('Add participant'))
-        self.__additionButton.clicked.connect(self._addParticipant)
+        QObject.connect(self.__additionButton, SIGNAL('clicked()'),
+                self._addParticipant)
 
     def _initComponents(self):
         '''Initialize components'''
@@ -1050,60 +1070,37 @@ class NetworkAuthorizationDialog(QMessageBox):
         self.__mapButton = {acceptButton: 'accept', denyButton: 'deny', blockButton: 'block'}
 
 
-class NetworkServiceDialog(CommonDialog):
+class NetworkSettingsDialog(CommonDialog):
     '''
-    Abstract class for service dialog
+    Abstract class for network settings dialog
     @author: Oleg Kandaurov
     '''
-    readable('attributes')
+
     def __init__(self, parent = None):
         CommonDialog.__init__(self, parent)
-        self.setWindowTitle(self.tr('Participation behaviour'))
-        self.__attributes = dict()
+        self.setWindowTitle(self.tr('Network settings'))
         self._createComponents()
         self._initComponents()
 
     def _createComponents(self):
         '''Create components'''
         #pylint: disable = W0201
-        self.__openModelRadio = QRadioButton(self.tr('Open'))
-        self.__authModelRadio = QRadioButton(self.tr('Interactive'))
-        self.__rosterModelRadio = QRadioButton(self.tr('Contact list'))
-        self.__whitelistModelRadio = QRadioButton(self.tr('White list'))
-        self.__accessModelGroup = QButtonGroup()
-        self.__accessModelGroup.addButton(self.__openModelRadio)
-        self.__accessModelGroup.addButton(self.__rosterModelRadio)
-        self.__accessModelGroup.addButton(self.__authModelRadio)
-        self.__accessModelGroup.addButton(self.__whitelistModelRadio)
-        self.__mapAccessModels = {'open': self.__openModelRadio,
-                'roster': self.__rosterModelRadio, 'authorize': self.__authModelRadio,
-                'whitelist': self.__whitelistModelRadio}
-        self.__mapAccessModels[settings.get('accessModel')].setChecked(True)
-        self.__readOnlyBinder = CheckBoxBinder('readOnly', settings.get('readOnly'))
-        self.__readOnlyBinder.setText(self.tr('Read only'))
+        self.__settings = SettingsCreator(['accessModel', 'publishModel', 'defaultAffiliation',
+            'presentationMode'])
+        self.__settingsWidget = QWidget()
+        self.__settingsWidget.setLayout(self.__settings.getLayout())
 
     def _initComponents(self):
         '''Create components'''
         mainLayout = QVBoxLayout()
-        accessModelGroupLayout = QGridLayout()
-        accessModelGroupLayout.addWidget(self.__openModelRadio, 0, 0, 1, 1)
-        accessModelGroupLayout.addWidget(self.__rosterModelRadio, 0, 1, 1, 1)
-        accessModelGroupLayout.addWidget(self.__authModelRadio, 1, 0, 1, 1)
-        accessModelGroupLayout.addWidget(self.__whitelistModelRadio, 1, 1, 1, 1)
-        accessModelGroupBox = QGroupBox(self.tr('Trust level'))
-        accessModelGroupBox.setLayout(accessModelGroupLayout)
-        mainLayout.addWidget(accessModelGroupBox)
-        mainLayout.addWidget(self.__readOnlyBinder)
+        mainLayout.addWidget(self.__settingsWidget)
         mainLayout.addWidget(self._buttonBox)
         self.setLayout(mainLayout)
 
     def _accept(self):
-        '''Hide the dialog, update attributes'''
-        self.__attributes = self.__readOnlyBinder.retrieve()
-        for model, button in self.__mapAccessModels.iteritems():
-            if button.isChecked():
-                self.__attributes['accessModel'] = model
-        settings.set(**self.__attributes)
+        '''Apply dialog'''
+        for setting in self.__settings.retrieve():
+            settings.set(**setting)
         QDialog.accept(self)
 
 
@@ -1133,7 +1130,7 @@ class NetworkClientDialog(CommonDialog):
         self.__serviceBinder = LineEditBinder('serviceJid', settings.get('serviceJid'),
                 QRegExp(VALID_JID))
         self.__serviceLabel = QLabel(self.tr('Service JID:'))
-        self.__view.clicked.connect(self._itemClicked)
+        QObject.connect(self.__view, SIGNAL('clicked()'), self._itemClicked)
 
     def _initComponents(self):
         '''Create components'''
@@ -1179,34 +1176,83 @@ class NetworkConnectionDialog(CommonDialog):
         #pylint: disable = W0201
         self.__userBinder = LineEditBinder('userJid', settings.get('userJid'),
                 QRegExp(VALID_JID))
-        self.__userLabel = QLabel(self.tr('User JID:'))
+        self.__userLabel = QLabel(self.tr('Jabber ID:'))
         self.__passwordEdit = QLineEdit()
         self.__passwordEdit.setEchoMode(QLineEdit.Password)
         password = settings.get('password')
         if password is not None:
             self.__passwordEdit.setText(password)
         self.__passwordLabel = QLabel(self.tr('Password:'))
+        self.__customSetting = QCheckBox(self.tr('Custom host/port'))
+        self.__customSetting.setChecked(False)
+        self.__hostLabel = QLabel(self.tr('Host:'))
+        self.__hostEdit = QLineEdit()
+        self.__hostEdit.setEnabled(False)
+        self.__portLabel = QLabel(self.tr('Port:'))
+        self.__portEdit = QSpinBox()
+        self.__portEdit.setEnabled(False)
+        self.__portEdit.setRange(0, 65535)
+        self.__customSetting.toggled.connect(self._adjustConnectionSettings)
 
     def _initComponents(self):
         '''Create components'''
+        tabWidget = QTabWidget()
+        QObject.connect(tabWidget, SIGNAL('currentChanged()'),
+                self._adjustConnectionSettings)
         mainLayout = QVBoxLayout()
-        formLayout = QFormLayout()
-        formLayout.addRow(self.__userLabel, self.__userBinder)
-        formLayout.addRow(self.__passwordLabel, self.__passwordEdit)
-        credentialsGroupBox = QGroupBox(self.tr('Credentials'))
-        credentialsGroupBox.setLayout(formLayout)
-        mainLayout.addWidget(credentialsGroupBox)
+        credentialsFormLayout = QFormLayout()
+        credentialsFormLayout.addRow(self.__userLabel, self.__userBinder)
+        credentialsFormLayout.addRow(self.__passwordLabel, self.__passwordEdit)
+        credentialsWidget = QWidget()
+        credentialsWidget.setLayout(credentialsFormLayout)
+        connectionFormLayout = QFormLayout()
+        connectionFormLayout.addRow(self.__customSetting)
+        connectionFormLayout.addRow(self.__hostLabel, self.__hostEdit)
+        connectionFormLayout.addRow(self.__portLabel, self.__portEdit)
+        connectionWidget = QWidget()
+        connectionWidget.setLayout(connectionFormLayout)
+        tabWidget.addTab(credentialsWidget, self.tr('Account'))
+        tabWidget.addTab(connectionWidget, self.tr('Connection'))
+        mainLayout.addWidget(tabWidget)
         mainLayout.addWidget(self._buttonBox)
         self.setLayout(mainLayout)
+        self.__userBinder.setFocus()
+        self.__userBinder.selectAll()
+
+    def _adjustConnectionSettings(self):
+        '''Adjust connection settings'''
+        isCustom = self.__customSetting.isChecked()
+        self.__hostEdit.setEnabled(isCustom)
+        self.__portEdit.setEnabled(isCustom)
+        if not isCustom:
+            if not self.__userBinder.isValid():
+                domain = ''
+            else:
+                _, domain = self.__userBinder.value.rsplit('@', 1)
+            self.__portEdit.setValue(settings.get('defaultXMPPPort'))
+            self.__hostEdit.setText(domain)
+
+    def _isValid(self):
+        '''Check whether all data in dialog are valid or not'''
+        if not self.__userBinder.isValid():
+            _showJIDErrorDialog(self)
+            return False
+        if not QRegExp(VALID_HOST).exactMatch(self.__hostEdit.text()):
+            QMessageBox.warning(self, self.tr('Incorrect hostname'),
+                    self.tr('Please, enter correct hostname'))
+            return False
+        return True
 
     def _accept(self):
         '''Hide the dialog, update attributes'''
-        if not self.__userBinder.isValid():
-            _showJIDErrorDialog(self)
+        self._adjustConnectionSettings()
+        if not self._isValid():
             return
         self.__attributes = self.__userBinder.retrieve()
         settings.set(**self.__attributes)
-        self.__attributes['password'] = self.__passwordEdit.text()
+        self.__attributes['password'] = unicode(self.__passwordEdit.text())
+        self.__attributes['host'] = unicode(self.__hostEdit.text())
+        self.__attributes['port'] = self.__portEdit.value()
         QDialog.accept(self)
 
 
@@ -1224,8 +1270,8 @@ class NetworkErrorDialog(QMessageBox):
         QMessageBox.__init__(self, parent)
         self.setIcon(QMessageBox.Warning)
         self.setWindowTitle(self.tr('Network error'))
-        self.setStandardButtons(QMessageBox.Ok);
-        self.setDefaultButton(QMessageBox.Ok);
+        self.setStandardButtons(QMessageBox.Ok)
+        self.setDefaultButton(QMessageBox.Ok)
 
     def showError(self, info, reason, details):
         '''
@@ -1242,3 +1288,56 @@ class NetworkErrorDialog(QMessageBox):
         self.setDetailedText(details)
         self.exec_()
 
+
+class EditHyperlinkDialog(CommonDialog):
+    '''
+    Dialog for node hyperlink editining
+    @author: Alexander Kulikov
+    '''
+    def __init__(self, node, mapDir, parent = None):
+        '''
+        Initialize with specific node
+        @type node: Node
+        @type mapDir: string
+        @type parent: QWidget
+        '''
+        CommonDialog.__init__(self, parent)
+        self.setWindowTitle(self.tr('Hyperlink editor'))
+        self.__node = node.clone()
+        self.__newAttributes = {}
+        self._createComponents()
+        self._initComponents()
+        self._initDialog()
+
+    def _createComponents(self):
+        '''Create components'''
+        #pylint: disable = W0201
+        self._urlPath = QLineEdit(self)
+
+    def _initDialog(self):
+        '''Init dialog'''
+        if self.__node.link:
+            self._urlPath.setText(self.__node.link)
+
+    def _initComponents(self):
+        '''Form initialization'''
+        layout = QGridLayout()
+        layout.addWidget(self._urlPath, 0, 0)
+        layout.addWidget(self._buttonBox, 1, 0)
+        self.setLayout(layout)
+
+    def _accept(self):
+        if self._urlPath.text():
+            self.__newAttributes = {'link': unicode(self._urlPath.text())}
+        else:
+            self.__newAttributes = {'link': None}
+        QDialog.accept(self)
+
+    def attributes(self):
+        '''
+        Get dict of modified attributes with new values
+
+        @return: new attribute values
+        @rtype: dict
+        '''
+        return self.__newAttributes
