@@ -25,9 +25,9 @@ View delegates for core objects
 '''
 
 import math
-from PySide.QtCore import SIGNAL, QRectF, QPointF, Qt
-from PySide.QtGui import QGraphicsScene, QGraphicsItem, QAction, QPen, \
-QPainterPath, QPolygonF, QIcon
+from PyQt4.QtCore import SIGNAL, QRectF, QPointF, Qt, QObject
+from PyQt4.QtGui import QGraphicsScene, QGraphicsItem, QAction, QPen, \
+QPainterPath, QPolygonF, QIcon, QTextDocument
 import hivemind.resource as resource
 import hivemind.settings as settings
 from hivemind.attribute import readable, writable
@@ -44,8 +44,7 @@ class MindMapDelegate(QGraphicsScene):
     @authors: Andrew Golovchenko, Andrew Vasilev, Alexander Kulikov
     '''
 
-    readable('rootDelegate', 'mindMap', 'selectLeftNodeAction', 'selectRightNodeAction',
-             'selectUpNodeAction', 'selectDownNodeAction', 'selectRootNodeAction')
+    readable('rootDelegate', 'mindMap')
     writable('currentLayerDepth')
 
     def _getCurrentNode(self):
@@ -59,18 +58,21 @@ class MindMapDelegate(QGraphicsScene):
         '''
         if self.__nodeToDelegate.has_key(self.__currentNode):
             self.__nodeToDelegate[self.__currentNode].selected = False
+
         self.__currentNode = currentNode
         delegate = self.__nodeToDelegate[currentNode]
         delegate.selected = True
         delegate.ensureVisible(delegate.textBoundingRect())
+        self.__currentLayerDepth = currentNode.depth()
         self.emit(SIGNAL('currentNodeChanged'), self.__currentNode)
 
     currentNode = property(_getCurrentNode, _setCurrentNode, None, 'Current node')
 
-    def __init__(self):
+    def __init__(self, actionBag):
         '''Initialize empty mind map'''
         QGraphicsScene.__init__(self)
         settings.connect(self._settingsChanged)
+        self.__actionBag = actionBag
         self.__nodeToDelegate = {}
         self.__rootDelegate = None
         self.__currentLayerDepth = None
@@ -83,11 +85,11 @@ class MindMapDelegate(QGraphicsScene):
                                  'localAttributeChanged' : self.nodeBoundsChanged,
                                  'inheritedAttributeChanged' : self.inheritedAttributeChanged}
         self.setMap(MindMap())
-        self.__selectLeftNodeAction = self.createAction(None, self._selectLeftNode, 'Left')
-        self.__selectRightNodeAction = self.createAction(None, self._selectRightNode, 'Right')
-        self.__selectUpNodeAction = self.createAction(None, self._selectUpNode, 'Up')
-        self.__selectDownNodeAction = self.createAction(None, self._selectDownNode, 'Down')
-        self.__selectRootNodeAction = self.createAction(None, self._selectRootNode, 'Escape')
+        self.__actionBag.selectLeftNodeAction.triggered.connect(self._selectLeftNode)
+        self.__actionBag.selectRightNodeAction.triggered.connect(self._selectRightNode)
+        self.__actionBag.selectUpNodeAction.triggered.connect(self._selectUpNode)
+        self.__actionBag.selectDownNodeAction.triggered.connect(self._selectDownNode)
+        self.__actionBag.selectRootNodeAction.triggered.connect(self._selectRootNode)
 
     def createAction(self, text, handler, shortcut = None, icon = None):
         '''
@@ -101,7 +103,7 @@ class MindMapDelegate(QGraphicsScene):
         action = QAction(text, self)
         if shortcut: action.setShortcut(shortcut)
         if icon: action.setIcon(QIcon.fromTheme(icon))
-        if handler: action.connect(SIGNAL('triggered()'), handler)
+        if handler: QObject.connect(action, SIGNAL('triggered()'), handler)
         return action
 
     def addNodeDelegate(self, node):
@@ -158,6 +160,8 @@ class MindMapDelegate(QGraphicsScene):
         '''
         notifyType = args.get('type')
         self.__signalHandlers.get(notifyType)(*values)
+        if notifyType in ['childNodeAdded', 'childNodeRemoved', 'nodeMoved', 'inheritedAttributeChanged']:
+            self.emit(SIGNAL('curMapChanged'))
 
     def removeNodeDelegate(self, node):
         '''
@@ -485,14 +489,6 @@ class MindMapDelegate(QGraphicsScene):
                     self.currentNode = nodes[index + 1]
                 return
 
-    def onItemClicked(self, rightButton, item, parent, childLocation):
-        '''
-        Update scene attributes according to clicked item
-        '''
-        if item is not None:
-            self.currentNode = item.node
-            self.__currentLayerDepth = item.node.depth()
-
 
 class NodeDelegate(QGraphicsItem):
     '''
@@ -516,6 +512,19 @@ class NodeDelegate(QGraphicsItem):
 
     selected = property(_isSelected, _setSelected, None, 'Selected flag')
 
+    def _isHighlighted(self):
+        '''Get highlighted flag'''
+        return self.__highlighted
+
+    def _setHighlighted(self, highlighted):
+        '''
+        Set the highlighted flag
+        @type highlighted: boolean
+        '''
+        self.__highlighted = highlighted
+
+    highlighted = property(_isHighlighted, _setHighlighted, None, 'Highlighted flag')
+
     def __init__(self, node, parent = None):
         '''
         Initialize delegate with specified node
@@ -527,6 +536,7 @@ class NodeDelegate(QGraphicsItem):
         '''
         self.__node = node
         self.__verticalSpace = 0
+        self.__highlighted = False
         self.__selected = False
         self.__positionChanged = True
         self.__linkPoint = None
@@ -546,6 +556,9 @@ class NodeDelegate(QGraphicsItem):
     __ICON_SIZE = 16
     '''Icon width and height'''
 
+    __LINK_SIZE = 24
+    '''Link icon width and height'''
+
     __TEXT_MARGIN = 4.0
     '''Default node content margin. It is never changed manually'''
 
@@ -717,6 +730,8 @@ class NodeDelegate(QGraphicsItem):
         edgeStyle = self.__node.effectiveEdgeStyle
         # node background color
         bgColor = self.__node.effectiveBackgroundColor
+        if self.__highlighted:
+            bgColor = settings.get('defaultSelectedNodeBackgroundColor').lighter(60)
         if self.__selected:
             bgColor = settings.get('defaultSelectedNodeBackgroundColor')
         painter.setPen(QPen(edgeColor, 1.0))
@@ -838,26 +853,40 @@ class NodeDelegate(QGraphicsItem):
         boundRect = self.textBoundingRect()
         return QPointF((boundRect.width() -
                         len(self.__node.icons) * NodeDelegate.__ICON_SIZE -
-                        self.__node.textSize().width()) / 2.0,
+                        self.__node.textSize().width() -
+                        (NodeDelegate.__LINK_SIZE if self.__node.link else 0)) / 2.0,
                         (boundRect.height() - self.__node.textSize().height()) / 2.0)
 
     def _paintNodeContent(self, painter):
         '''Paint internal content of the node'''
         painter.save()
         if not self.parentItem(): painter.translate(self._rootTextPosition())
+        # folded mark offset
+        if self.__node.folded and self.__node.left:
+            painter.translate(NodeDelegate.__FOLD_MARK_SIZE, 0)
         # paint icons
         xIconPos = NodeDelegate.__TEXT_MARGIN
         yIconPos = (self.__node.textSize().height() - NodeDelegate.__ICON_SIZE) / 2.0
         for icon in self.__node.icons:
-            painter.drawImage(QPointF(xIconPos, yIconPos), resource.getIconByName(icon))
+            svgIcon = resource.getSvgIconByName(icon)
+            if svgIcon:
+                svgIcon.render(painter, QRectF(xIconPos, yIconPos,
+                        NodeDelegate.__ICON_SIZE, NodeDelegate.__ICON_SIZE))
+            else:
+                painter.drawImage(QPointF(xIconPos, yIconPos),
+                        resource.getIconByName(icon))
             xIconPos += NodeDelegate.__ICON_SIZE
-        # folded mark offset
-        if self.__node.folded and self.__node.left:
-            painter.translate(NodeDelegate.__FOLD_MARK_SIZE, 0)
         # paint text content
         if xIconPos > NodeDelegate.__TEXT_MARGIN:
             painter.translate(xIconPos, 0)
         self.__node.paint(painter)
+        # paint link icon
+        if self.__node.link:
+            xIconPos = self.__node.textSize().width()
+            yIconPos = (self.__node.textSize().height() - NodeDelegate.__LINK_SIZE) / 2.0
+            painter.drawImage(QRectF(xIconPos, yIconPos, NodeDelegate.__LINK_SIZE,
+                    NodeDelegate.__LINK_SIZE),
+                    resource.getImage('link').toImage())
         painter.restore()
 
     def _calculateTextBoundingRect(self):
@@ -867,6 +896,10 @@ class NodeDelegate(QGraphicsItem):
             size.setWidth(size.width() + NodeDelegate.__TEXT_MARGIN
                     + len(self.__node.icons) * NodeDelegate.__ICON_SIZE)
             size.setHeight(max(size.height(), NodeDelegate.__ICON_SIZE))
+        if self.__node.link:
+            size.setWidth(size.width() + NodeDelegate.__LINK_SIZE +
+                    NodeDelegate.__TEXT_MARGIN)
+            size.setHeight(max(size.height(), NodeDelegate.__LINK_SIZE))
         if not self.parentItem():
             self.__textBoundingRect = self._rootBoundingRect(size)
             return
@@ -1070,7 +1103,6 @@ class NodeLocation(object):
 
     @author: Oleg Kandaurov
     '''
-    readable('right', 'top', 'position')
     writable('right', 'top', 'position')
 
     def __init__(self, right = None, top = None, position = None):
@@ -1097,3 +1129,129 @@ class NodeLocation(object):
         '''
         return self.__position is not None or self.__right is not None
 
+
+class NodeFinder(QObject):
+    '''
+    Node finder
+    @author: Eldar Mamedov
+    '''
+    def __init__(self, controller):
+        '''
+        @type controller: MainWindowController
+        '''
+        QObject.__init__(self)
+        self.__foundNodes = []
+        self.__index = -1
+        self.__controller = controller
+        self.__findText = None
+        self.__caseSensitive = False
+        self.__highlight = False
+        QObject.connect(controller.mindMapScene, SIGNAL('curMapChanged'), self.refresh)
+        QObject.connect(controller.mindMapScene, SIGNAL('mapChanged'), self.refresh)
+
+    def prevFoundNode(self):
+        '''
+        Select previous found node
+        '''
+        if len(self.__foundNodes):
+            self.__index -= 1
+            if self.__index < 0:
+                self.__index = len(self.__foundNodes) - 1
+            foundNode = self.__foundNodes[self.__index]
+            self.__controller.showNode(foundNode)
+            self.__controller.mindMapScene.currentNode = foundNode
+        self._status()
+
+    def nextFoundNode(self):
+        '''
+        Select next found node
+        '''
+        if len(self.__foundNodes):
+            self.__index += 1
+            if self.__index == len(self.__foundNodes):
+                self.__index = 0
+            foundNode = self.__foundNodes[self.__index]
+            self.__controller.showNode(foundNode)
+            self.__controller.mindMapScene.currentNode = foundNode
+        self._status()
+
+    def highlightNodes(self, highlight = False):
+        '''
+        Highlight found nodes
+        @type highlight: bool
+        '''
+        self.__highlight = highlight
+        for node in self.__foundNodes:
+            nodeDelegate = self.__controller.mindMapScene.getDelegate(node)
+            if nodeDelegate is None: continue
+            if highlight:
+                nodeDelegate.highlighted = True
+            else:
+                nodeDelegate.highlighted = False
+        self.__controller.mindMapScene.update()
+
+    def findNodes(self, findText = None, caseSensitive = False, highlight = False):
+        '''
+        Find nodes
+        @type findText: str
+        @type caseSensitive: bool
+        @type highlight: bool
+        '''
+        def find(node, depth):
+            '''
+            Recursive function to find nodes
+            @type node: Node
+            @type depth: int
+            '''
+            if depth not in nodeMap:
+                nodeMap[depth] = []
+            text.setHtml(node.text)
+            if caseSensitive:
+                if findText in text.toPlainText():
+                    nodeMap[depth].append(node)
+            else:
+                if findText.toLower() in text.toPlainText().toLower():
+                    nodeMap[depth].append(node)
+            for childrenNode in node.children:
+                if childrenNode.left:
+                    find(childrenNode, depth - 1)
+                else:
+                    find(childrenNode, depth + 1)
+        if self.__highlight: self.highlightNodes(False)
+        self.__foundNodes = []
+        self.__index = -1
+        nodeMap = {}
+        text = QTextDocument()
+        self.__findText = findText
+        self.__caseSensitive = caseSensitive
+        if not len(findText): return
+        find(self.__controller.mindMapScene.rootDelegate.node, 0)
+        keys = nodeMap.keys()
+        keys.sort()
+        for key in keys:
+            self.__foundNodes.extend(nodeMap[key])
+        if highlight: self.highlightNodes(highlight)
+        self._status()
+
+    def refresh(self):
+        '''
+        Refresh list of found nodes
+        '''
+        self.findNodes(self.__findText, self.__caseSensitive, self.__highlight)
+
+    def _status(self):
+        '''
+        Generates signal about status
+        '''
+        if not len(self.__foundNodes):
+            self.emit(SIGNAL('statusChanged'), 'not_found')
+            return
+        else:
+            if self.__index == len(self.__foundNodes) - 1:
+                self.emit(SIGNAL('statusChanged'), 'last')
+                return
+            if self.__index == 0:
+                self.emit(SIGNAL('statusChanged'), 'first')
+                return
+        self.emit(SIGNAL('statusChanged'), 'none')
+
