@@ -26,22 +26,24 @@ GUI factory
 @author: Alexander Kulikov, Ilya Paramonov
 '''
 
+from PyQt4.QtGui import QFileDialog
 import hivemind.sysconfig as sysconfig
 from hivemind.dialogs import SideSelectionDialog, NetworkAuthorizationDialog, \
-        NetworkErrorDialog
+        NetworkErrorDialog, EditHyperlinkDialog
 
 if sysconfig.PLATFORM == 'maemo':
     from hivemind.maemo.main_window import MaemoMainWindow
     from hivemind.maemo.dialogs import MaemoEditNodeDialog, MaemoEditLabelDialog
     from hivemind.maemo.dialogs import MaemoIconDialog, MaemoAboutDialog, MaemoEditEdgeDialog
     from hivemind.maemo.dialogs import MaemoSettingsDialog, MaemoNetworkConnectionDialog
-    from hivemind.maemo.dialogs import MaemoAffiliationDialog, MaemoExtendedMenuDialog
-    from hivemind.maemo.dialogs import MaemoNetworkServiceDialog, MaemoNetworkClientDialog
+    from hivemind.maemo.dialogs import MaemoAffiliationDialog, MaemoFindToolBar
+    from hivemind.maemo.dialogs import MaemoNetworkSettingsDialog, MaemoNetworkClientDialog
 else:
     from hivemind.desktop.main_window import DesktopMainWindow
     from hivemind.desktop.dialogs import DesktopEditNodeDialog, DesktopEditLabelDialog
-    from hivemind.desktop.dialogs import DesktopIconDialog, DesktopAboutDialog
-    from hivemind.desktop.dialogs import DesktopEditEdgeDialog, DesktopNetworkServiceDialog
+    from hivemind.desktop.dialogs import DesktopIconDialog, DesktopAboutDialog, \
+        DesktopFindToolBar
+    from hivemind.desktop.dialogs import DesktopEditEdgeDialog, DesktopNetworkSettingsDialog
     from hivemind.desktop.dialogs import DesktopSettingsDialog, DesktopNetworkConnectionDialog
     from hivemind.desktop.dialogs import DesktopAffiliationDialog, DesktopNetworkClientDialog
 
@@ -103,6 +105,15 @@ def createAboutDialog():
         return MaemoAboutDialog(__defaultParent)
     return DesktopAboutDialog(__defaultParent)
 
+def createFindToolBar(controller):
+    '''
+    Factory for creating families of related find dialogs
+    @type controller: MainWindowController
+    '''
+    if sysconfig.PLATFORM == 'maemo':
+        return MaemoFindToolBar(controller, __defaultParent)
+    return DesktopFindToolBar(controller, __defaultParent)
+
 def createSettingsDialog():
     '''
     Factory for creating families of related settings dialogs
@@ -142,13 +153,13 @@ def createNetworkAuthorizationDialog(requestor):
     '''
     return NetworkAuthorizationDialog(requestor, __defaultParent)
 
-def createNetworkServiceDialog():
+def createNetworkSettingsDialog():
     '''
-    Factory for creating families of related service dialogs
+    Factory for creating families of related network settings dialogs
     '''
     if sysconfig.PLATFORM == 'maemo':
-        return MaemoNetworkServiceDialog(__defaultParent)
-    return DesktopNetworkServiceDialog(__defaultParent)
+        return MaemoNetworkSettingsDialog(__defaultParent)
+    return DesktopNetworkSettingsDialog(__defaultParent)
 
 def createNetworkClientDialog(rosterModel):
     '''
@@ -171,3 +182,30 @@ def createNetworkErrorDialog():
     Factory for creating families of network error dialogs
     '''
     return NetworkErrorDialog(__defaultParent)
+
+def createEditHyperlinkDialog(node, curdir):
+    '''
+    Factory for creating families of hyperlink edit dialogs
+    @type node: Node
+    @type curdir: string
+    '''
+    return EditHyperlinkDialog(node, curdir, __defaultParent)
+
+def invokeSaveFileDialog(title, baseDirectory, filter, selectedFilter):
+    '''
+    Factory for creating save file dialog
+    @param title: dialog title
+    @type title: str
+    @param baseDirectory: base save file directory
+    @type baseDirectory: str
+    @param filter: file type filter
+    @type filter: str
+    @param selectedFilter: default filter selected, may be None
+    @type selectedFilter: str
+    @return: selected file name and active filter
+    @rtype: [QString, QString]
+    '''
+    if sysconfig.PLATFORM == 'maemo':
+        baseDirectory = '/'
+    return QFileDialog.getSaveFileNameAndFilter(defaultParent(), title, baseDirectory, filter,
+            selectedFilter)
