@@ -25,34 +25,36 @@
 Main window classes
 '''
 
+'''Install and start QtReactor'''
+import sys
+from PyQt4.QtGui import QApplication
+app = QApplication(sys.argv)
+from qt4reactor import qt4reactor
+qt4reactor.install()
+from hivemind.network import network_core
+network_core.startReactor()
+
 import getopt
 import os
 import re
 from twisted.python import log
-from PySide.QtGui import QActionGroup, QGraphicsBlurEffect, QMessageBox, QUndoStack, \
-QFileDialog, QApplication, QUndoCommand
-from PySide.QtCore import QObject, SIGNAL, QTimer, QLocale, QTranslator, QLibraryInfo
+from PyQt4.QtGui import QActionGroup, QGraphicsBlurEffect, QMessageBox, QUndoStack, \
+QFileDialog, QPixmap, QPainter, QAction, QDesktopServices, QPrintDialog, QPrinter
+from PyQt4.QtCore import QObject, SIGNAL, QTimer, QLocale, QTranslator, QLibraryInfo, \
+Qt, QRectF, QUrl
 import hivemind.gui_factory as gui_factory
 import hivemind.parser as parser
 import hivemind.settings as settings
 import hivemind.sysconfig as sysconfig
 from hivemind.action_bag import ActionBag
-from hivemind.attribute import writable, adaptCallable, sys
+from hivemind.attribute import writable, adaptCallable
 from hivemind.commands import AddNodeCommand, StateModificationCommand, RemoveNodeCommand, \
-MoveNodeCommand
+MoveNodeCommand, ActionCommand, CurrentNodeCommand, HivemindCommand
 from hivemind.core import MindMap
 from hivemind.gui_delegates import MindMapDelegate, NodeLocation
 from hivemind.gui_widgets import readable
+from hivemind.robots import SmartConferenceRobot
 
-# Install qt4reactor as main twisted framework reactor
-from twisted.internet import main
-from qt4reactor.qt4reactor import QTReactor
-app = QApplication(sys.argv)
-global reactor
-reactor = QTReactor()
-main.installReactor(reactor)
-
-from hivemind.network.network_core import NetworkController
 
 class MainWindowController(QObject):
     '''
@@ -61,6 +63,7 @@ class MainWindowController(QObject):
     '''
 
     writable('mode')
+    readable('mindMapScene')
 
     MODE_NORMAL = 'normal'
     '''All actions are parsed as normal'''
@@ -71,22 +74,24 @@ class MainWindowController(QObject):
     def __init__(self):
         QObject.__init__(self)
         '''Init widgets and configure application'''
+        self.__findWidget = None
         self.__actionBag = ActionBag(self)
         self.__mode = self.MODE_NORMAL
         self._createActions()
         self._createStateModificationActions()
-        self.__mindMapScene = MindMapDelegate()
-        self.connect(self.__mindMapScene, SIGNAL('currentNodeChanged'),
+        self.__mindMapScene = MindMapDelegate(self.__actionBag)
+        QObject.connect(self.__mindMapScene, SIGNAL('currentNodeChanged'),
                      adaptCallable(self._setActionsState))
         self.__mainWindow = gui_factory.createMainWindow(self.__actionBag, self.__mindMapScene)
-        mindMapController = MindMapController(self.__mindMapScene, self.__actionBag)
-        self.__networkController = NetworkController(mindMapController, self.__actionBag)
-        self.connect(self.__mainWindow.mindMapView, SIGNAL('itemClicked'),
-            adaptCallable(self.__mindMapScene.onItemClicked))
-        self.connect(self.__mainWindow.mindMapView, SIGNAL('itemClicked'),
-            adaptCallable(self.onItemClicked))
-        self.connect(self.__mainWindow, SIGNAL('closeEvent'), self._closeEvent)
-        _configureApplication(mindMapController)
+        self.__mindMapController = MindMapController(self.__mindMapScene, self.__actionBag)
+        self.__networkController = network_core.NetworkController(
+                self.__mindMapController, self.__actionBag)
+        self.__smartConferenceRobot = SmartConferenceRobot(self.__mindMapScene)
+        QObject.connect(self.__actionBag.startSmartConferenceRobot, SIGNAL('triggered()'), self.__smartConferenceRobot.start)
+        QObject.connect(self.__actionBag.stopSmartConferenceRobot, SIGNAL('triggered()'), self.__smartConferenceRobot.stop)
+        QObject.connect(self.__mainWindow.mindMapView, SIGNAL('itemClicked'), adaptCallable(self.onItemClicked))
+        QObject.connect(self.__mainWindow, SIGNAL('closeEvent'), self._closeEvent)
+        _configureApplication(self.__mindMapController)
         self.__mainWindow.showMaximized()
 
     def _setActionsState(self, currentNode):
@@ -114,12 +119,14 @@ class MainWindowController(QObject):
         self.__actionBag.addNodeAction.setEnabled(self.__mode == self.MODE_NORMAL)
         self.__actionBag.editNodeIconsAction.setEnabled(self.__mode == self.MODE_NORMAL)
         self.__actionBag.editNodeAction.setEnabled(self.__mode == self.MODE_NORMAL)
-        self.__mindMapScene.selectRootNodeAction.setEnabled(self.__mode == self.MODE_NORMAL)
+        self.__actionBag.selectRootNodeAction.setEnabled(self.__mode == self.MODE_NORMAL)
         self.__actionBag.cancelTransferModeAction.setEnabled(self.__mode == self.MODE_TRANSFER)
         self.__actionBag.putNodeAction.setEnabled(self.__mode == self.MODE_TRANSFER)
         self.__actionBag.putNodeBelowAction.setEnabled(self.__mode == self.MODE_TRANSFER)
         self.__actionBag.putNodeAboveAction.setEnabled(self.__mode == self.MODE_TRANSFER)
         self.__actionBag.showContextMenuAction.setEnabled(self.__mode == self.MODE_NORMAL)
+        self.__actionBag.openHyperlinkAction.setEnabled(
+                self.__mode == self.MODE_NORMAL and currentNode.link is not None)
 
     def _connectStateModificationAction(self, action, alteredAttributes, checkable = True):
         '''
@@ -132,7 +139,7 @@ class MainWindowController(QObject):
         def handler():
             self.createStateModificationCommand(
                     self.__mindMapScene.currentNode, alteredAttributes)
-        action.connect(SIGNAL('triggered()'), handler)
+        QObject.connect(action, SIGNAL('triggered()'), handler)
         action.setCheckable(checkable)
 
     def _createStateModificationActions(self):
@@ -179,45 +186,41 @@ class MainWindowController(QObject):
 
     def _createActions(self):
         '''Create actions'''
-        self.__actionBag.editEdgeAction.connect(SIGNAL('triggered()'), self.editEdge)
-        self.__actionBag.editLabelAction.connect(SIGNAL('triggered()'), self.editLabel)
-        self.__actionBag.editNodeAction.connect(SIGNAL('triggered()'), self.editNode)
-        self.__actionBag.editNodeIconsAction.connect(SIGNAL('triggered()'), self.editNodeIcons)
-        self.__actionBag.foldNodeAction.connect(SIGNAL('triggered()'), self.foldNode)
-        self.__actionBag.addNodeAction.connect(SIGNAL('triggered()'), self.addNode)
-        self.__actionBag.addSiblingNodeBelowAction.connect(SIGNAL('triggered()'),
-            self._addSiblingNodeBelowAction)
-        self.__actionBag.addSiblingNodeAboveAction.connect(SIGNAL('triggered()'),
-            self._addSiblingNodeAboveAction)
-        self.__actionBag.removeNodeAction.connect(SIGNAL('triggered()'), self.removeNode)
-        self.__actionBag.enterTransferModeAction.connect(SIGNAL('triggered()'),
-                self._startNodeMoving)
-        self.__actionBag.cancelTransferModeAction.connect(SIGNAL('triggered()'),
-                self._cancelNodeMoving)
-        self.__actionBag.putNodeAction.connect(SIGNAL('triggered()'), self._putNode)
-        self.__actionBag.putNodeBelowAction.connect(SIGNAL('triggered()'),
-                self._putNodeBelow)
-        self.__actionBag.putNodeAboveAction.connect(SIGNAL('triggered()'),
-                self._putNodeAbove)
-        self.__actionBag.undoAction.connect(SIGNAL('triggered()'), self._doUndo)
-        self.__actionBag.redoAction.connect(SIGNAL('triggered()'), self._doRedo)
-        self.__actionBag.aboutAction.connect(SIGNAL('triggered()'), self.onAbout)
-        self.__actionBag.exitAction.connect(SIGNAL('triggered()'), self._closeMainWindow)
-        self.__actionBag.settingsAction.connect(SIGNAL('triggered()'), self.onSettings)
-        self.__actionBag.showContextMenuAction.connect(SIGNAL('triggered()'),
-                self._virtualClick)
+        QObject.connect(self.__actionBag.editEdgeAction, SIGNAL('triggered()'), self.editEdge)
+        QObject.connect(self.__actionBag.editLabelAction, SIGNAL('triggered()'), self.editLabel)
+        QObject.connect(self.__actionBag.editNodeAction, SIGNAL('triggered()'), self.editNode)
+        QObject.connect(self.__actionBag.editNodeIconsAction, SIGNAL('triggered()'), self.editNodeIcons)
+        QObject.connect(self.__actionBag.foldNodeAction, SIGNAL('triggered()'), self.foldNode)
+        QObject.connect(self.__actionBag.addNodeAction, SIGNAL('triggered()'), self.addNode)
+        QObject.connect(self.__actionBag.addSiblingNodeBelowAction, SIGNAL('triggered()'), self._addSiblingNodeBelowAction)
+        QObject.connect(self.__actionBag.addSiblingNodeAboveAction, SIGNAL('triggered()'), self._addSiblingNodeAboveAction)
+        QObject.connect(self.__actionBag.removeNodeAction, SIGNAL('triggered()'), self.removeNode)
+        QObject.connect(self.__actionBag.enterTransferModeAction, SIGNAL('triggered()'), self._startNodeMoving)
+        QObject.connect(self.__actionBag.cancelTransferModeAction, SIGNAL('triggered()'), self._cancelNodeMoving)
+        QObject.connect(self.__actionBag.putNodeAction, SIGNAL('triggered()'), self._putNode)
+        QObject.connect(self.__actionBag.putNodeBelowAction, SIGNAL('triggered()'), self._putNodeBelow)
+        QObject.connect(self.__actionBag.putNodeAboveAction, SIGNAL('triggered()'), self._putNodeAbove)
+        QObject.connect(self.__actionBag.undoAction, SIGNAL('triggered()'), self._doUndo)
+        QObject.connect(self.__actionBag.redoAction, SIGNAL('triggered()'), self._doRedo)
+        QObject.connect(self.__actionBag.aboutAction, SIGNAL('triggered()'), self.onAbout)
+        QObject.connect(self.__actionBag.exitAction, SIGNAL('triggered()'), self._closeMainWindow)
+        QObject.connect(self.__actionBag.settingsAction, SIGNAL('triggered()'), self.onSettings)
+        QObject.connect(self.__actionBag.findAction, SIGNAL('triggered()'), self.onFind)
+        QObject.connect(self.__actionBag.showContextMenuAction, SIGNAL('triggered()'), self._virtualClick)
+        QObject.connect(self.__actionBag.editHyperlinkAction, SIGNAL('triggered()'), self.editHyperlink)
+        QObject.connect(self.__actionBag.openHyperlinkAction, SIGNAL('triggered()'), self.openHyperlink)
 
     def _doUndo(self):
         '''
         Record undo action
         '''
-        self._recordCommand('undo')
+        self._recordCommand(ActionCommand('undo'))
 
     def _doRedo(self):
         '''
         Record redo action
         '''
-        self._recordCommand('redo')
+        self._recordCommand(ActionCommand('redo'))
 
     def createStateModificationCommand(self, selectedNode, alteredAttributes):
         '''
@@ -257,7 +260,7 @@ class MainWindowController(QObject):
                     unmodifiedAttributes[key] = initialAttributeValues[key]
             self._recordCommand(StateModificationCommand(node, unmodifiedAttributes,
                     alteredAttributes))
-        dialog.connect(SIGNAL('accepted()'), _accept)
+        QObject.connect(dialog, SIGNAL('accepted()'), _accept)
         dialog.exec_()
 
     def _recordCommand(self, command):
@@ -273,6 +276,7 @@ class MainWindowController(QObject):
         self.__mainWindow.showMenu(True, self.__mindMapScene.currentNode)
 
     def onItemClicked(self, rightButton, item, parent, childLocation):
+        if item: self._recordCommand(CurrentNodeCommand(item.node))
         if self.__mode == MainWindowController.MODE_NORMAL:
             if item is not None:
                 self.__mainWindow.showMenu(rightButton, item)
@@ -285,34 +289,34 @@ class MainWindowController(QObject):
 
     def _addSiblingNodeBelowAction(self):
         '''Create and edit a sibling node, add it under selected node'''
-        if self.__mindMapScene.currentNode.parent is None:
+        parentNode = self.__mindMapScene.currentNode.parent
+        index = parentNode.children.index(self.__mindMapScene.currentNode)
+        if parentNode is None:
             return
         def _accept():
             '''Process accept action'''
-            newNode.setAttributes(left = self.__mindMapScene.currentNode.left)
             newNode.setAttributes(**editor.attributes())
-            parent = self.__mindMapScene.currentNode.parent
-            index = parent.children.index(self.__mindMapScene.currentNode)
-            self._recordCommand(AddNodeCommand(parent, newNode, index + 1))
-        newNode = self.__mindMapScene.currentNode.parent.createChildPrototype()
+            self._recordCommand(AddNodeCommand(parentNode, newNode, index + 1))
+        newNode = parentNode.createChildPrototype()
+        newNode.setAttributes(left = self.__mindMapScene.currentNode.left)
         editor = gui_factory.createEditNodeDialog(newNode)
-        editor.connect(SIGNAL('accepted()'), _accept)
+        QObject.connect(editor, SIGNAL('accepted()'), _accept)
         editor.exec_()
 
     def _addSiblingNodeAboveAction(self):
         '''Create and edit a sibling node, add it over selected node'''
-        if self.__mindMapScene.currentNode.parent is None:
+        parentNode = self.__mindMapScene.currentNode.parent
+        index = parentNode.children.index(self.__mindMapScene.currentNode)
+        if parentNode is None:
             return
         def _accept():
             '''Process accept action'''
-            newNode.setAttributes(left = self.__mindMapScene.currentNode.left)
             newNode.setAttributes(**editor.attributes())
-            parent = self.__mindMapScene.currentNode.parent
-            index = parent.children.index(self.__mindMapScene.currentNode)
-            self._recordCommand(AddNodeCommand(parent, newNode, index))
-        newNode = self.__mindMapScene.currentNode.parent.createChildPrototype()
+            self._recordCommand(AddNodeCommand(parentNode, newNode, index))
+        newNode = parentNode.createChildPrototype()
+        newNode.setAttributes(left = self.__mindMapScene.currentNode.left)
         editor = gui_factory.createEditNodeDialog(newNode)
-        editor.connect(SIGNAL('accepted()'), _accept)
+        QObject.connect(editor, SIGNAL('accepted()'), _accept)
         editor.exec_()
 
     def _startNodeMoving(self):
@@ -381,18 +385,37 @@ class MainWindowController(QObject):
         self.moveNode(self.__movingNode, parent.node, location)
         self._setActionsState(self.__mindMapScene.currentNode)
 
-    def onAbout(self):
+    @classmethod
+    def onAbout(cls):
         '''
         Process about action
         '''
         gui_factory.createAboutDialog().exec_()
 
-    def onSettings(self):
+    @classmethod
+    def onSettings(cls):
         '''
         Process settings action
         '''
         gui_factory.createSettingsDialog().exec_()
 
+    def onFind(self):
+        '''
+        Process find action
+        '''
+        if self.__findWidget is None:
+            self.__findWidget = gui_factory.createFindToolBar(self)
+            if sysconfig.PLATFORM == 'maemo':
+                self.__findWidget.move(self.__mainWindow.width() - self.__findWidget.width() + 10,
+                        self.__mainWindow.height() - self.__findWidget.height() + 10)
+            else:
+                self.__mainWindow.addToolBar(Qt.BottomToolBarArea, self.__findWidget)
+        if self.__findWidget.isVisible():
+            self.__findWidget.setVisible(False)
+        else:
+            self.__findWidget.setVisible(True)
+            self.__findWidget._findLineEdit.setFocus()
+
     def editEdge(self):
         '''Edit node's edge decorations and style'''
         dialog = gui_factory.createEditEdgeDialog(self.__mindMapScene.currentNode)
@@ -408,22 +431,40 @@ class MainWindowController(QObject):
         dialog = gui_factory.createIconDialog(self.__mindMapScene.currentNode)
         self._recordStateModificationCommand(self.__mindMapScene.currentNode, dialog)
 
-    def foldNode(self):
-        '''Fold node subtree'''
+    def showNode(self, node):
+        '''
+        Show node if it was invisible
+        @param node: node to show
+        @type node: Node
+        '''
+        nodeParent = node.parent
+        while nodeParent:
+            if nodeParent.folded: self.foldNode(nodeParent)
+            nodeParent = nodeParent.parent
+
+    def foldNode(self, node = None):
+        '''
+        Fold node subtree
+        @param node: node to fold
+        @type node: Node
+        '''
+        if node is None:
+            node = self.__mindMapScene.currentNode
         unmodifiedAttributes = {}
         alteredAttributes = {}
-        unmodifiedAttributes['folded'] = self.__mindMapScene.currentNode.folded
-        if self.__mindMapScene.currentNode.folded is None:
+        unmodifiedAttributes['folded'] = node.folded
+        if node.folded is None:
             alteredAttributes['folded'] = True
         else:
             alteredAttributes['folded'] = None
-        self._recordCommand(StateModificationCommand(self.__mindMapScene.currentNode,
-                unmodifiedAttributes, alteredAttributes))
+        self._recordCommand(StateModificationCommand(node, unmodifiedAttributes,
+                alteredAttributes))
 
     def addNode(self):
         '''
         Create and edit a child node, add it to selected node
         '''
+        parentNode = self.__mindMapScene.currentNode
         def _accept():
             '''Process accept action'''
             if self.__mindMapScene.currentNode.parent is None:
@@ -436,10 +477,10 @@ class MainWindowController(QObject):
             if self.__mindMapScene.currentNode.folded:
                 self.foldNode()
             childNode.setAttributes(**editor.attributes())
-            self._recordCommand(AddNodeCommand(self.__mindMapScene.currentNode, childNode))
-        childNode = self.__mindMapScene.currentNode.createChildPrototype()
+            self._recordCommand(AddNodeCommand(parentNode, childNode))
+        childNode = parentNode.createChildPrototype()
         editor = gui_factory.createEditNodeDialog(childNode)
-        editor.connect(SIGNAL('accepted()'), _accept)
+        QObject.connect(editor, SIGNAL('accepted()'), _accept)
         editor.exec_()
 
     def removeNode(self):
@@ -483,12 +524,24 @@ class MainWindowController(QObject):
         dialog = gui_factory.createEditLabelDialog(self.__mindMapScene.currentNode)
         self._recordStateModificationCommand(self.__mindMapScene.currentNode, dialog)
 
+    def editHyperlink(self):
+        '''Edit hyperlink'''
+        dialog = gui_factory.createEditHyperlinkDialog(self.__mindMapScene.currentNode,
+                self.__mindMapController.fileDirectory())
+        self._recordStateModificationCommand(self.__mindMapScene.currentNode, dialog)
+
+    def openHyperlink(self):
+        '''Open node's hyperlink'''
+        if self.__mindMapScene.currentNode.link:
+            QDesktopServices.openUrl(QUrl(self.__mindMapScene.currentNode.link))
+
     def _closeMainWindow(self):
         '''Close main window of the application'''
         self.__mainWindow.close()
 
     def _closeEvent(self):
         '''Close network subsystem'''
+        self.__smartConferenceRobot.stop()
         self.__networkController.terminateNetworkConnection()
 
 
@@ -513,13 +566,17 @@ class MindMapController(QObject):
         self.__autosaved = False
         self.__autosaveTimer = QTimer(self)
         self.__autosaveTimer.setSingleShot(False)
-        self.__autosaveTimer.connect(SIGNAL('timeout()'), self._backupMindMap)
+        QObject.connect(self.__autosaveTimer, SIGNAL('timeout()'), self._autosaveMindMap)
         settings.connect(self._updateAutosave)
         self._updateAutosave()
-        self.__actionBag.fileNewAction.connect(SIGNAL('triggered()'), self.onFileNew)
-        self.__actionBag.fileOpenAction.connect(SIGNAL('triggered()'), self.onFileOpen)
-        self.__actionBag.fileSaveAction.connect(SIGNAL('triggered()'), self.onFileSave)
-        self.__actionBag.fileSaveAsAction.connect(SIGNAL('triggered()'), self.onFileSaveAs)
+        QObject.connect(self.__actionBag.fileNewAction, SIGNAL('triggered()'), self.onFileNew)
+        QObject.connect(self.__actionBag.fileOpenAction, SIGNAL('triggered()'), self.onFileOpen)
+        QObject.connect(self.__actionBag.fileSaveAction, SIGNAL('triggered()'), self.onFileSave)
+        QObject.connect(self.__actionBag.fileSaveAsAction, SIGNAL('triggered()'), self.onFileSaveAs)
+        QObject.connect(self.__actionBag.exportToImageAction, SIGNAL('triggered()'), self.onExportToImage)
+        QObject.connect(self.__actionBag.printAction, SIGNAL('triggered()'), self.onPrint)
+        for action in self.__actionBag.recentFileActions:
+            QObject.connect(action, SIGNAL('triggered()'), self.openRecentFile)
 
     def _loadUndoStack(self, undoStack):
         '''
@@ -530,12 +587,9 @@ class MindMapController(QObject):
         self.__undoStack = undoStack
         self.__actionBag.undoAction.setEnabled(undoStack.canRedo())
         self.__actionBag.redoAction.setEnabled(undoStack.canUndo())
-        self.__undoStack.connect(SIGNAL('canUndoChanged(bool)'),
-            adaptCallable(self.__actionBag.undoAction.setEnabled))
-        self.__undoStack.connect(SIGNAL('canRedoChanged(bool)'),
-            adaptCallable(self.__actionBag.redoAction.setEnabled))
-        self.__undoStack.connect(SIGNAL('cleanChanged(bool)'),
-            adaptCallable(self._undoStackStateChanged))
+        QObject.connect(self.__undoStack, SIGNAL('canUndoChanged(bool)'), adaptCallable(self.__actionBag.undoAction.setEnabled))
+        QObject.connect(self.__undoStack, SIGNAL('canRedoChanged(bool)'), adaptCallable(self.__actionBag.redoAction.setEnabled))
+        QObject.connect(self.__undoStack, SIGNAL('cleanChanged(bool)'), adaptCallable(self._undoStackStateChanged))
 
     def loadUndoStackFromNetwork(self, stack):
         '''
@@ -546,19 +600,19 @@ class MindMapController(QObject):
         self._loadUndoStack(parser.deserializeCommandStack(stack,
                 self.__mindMapScene.mindMap))
 
-    def _backupMindMap(self):
-        '''Save backup of current map'''
+    def _autosaveMindMap(self):
+        '''Autosave current map'''
         if self.__undoStack.canUndo():
             if self.__fileName:
-                path = self._getBackupFilename(self.__fileName)
+                path = self._generateAutosaveFilename(self.__fileName)
                 parser.storeMindMap(path, self.__mindMapScene.mindMap)
 
-    def _removeBackup(self):
-        '''Remove backup of current map'''
-        backup = self._getBackupFilename(self.__fileName)
-        if backup is not None:
-            if os.path.isfile(backup):
-                os.remove(backup)
+    def _removeAutosave(self):
+        '''Remove autosave of current map'''
+        autosave = self._generateAutosaveFilename(self.__fileName)
+        if autosave is not None:
+            if os.path.isfile(autosave):
+                os.remove(autosave)
 
     def _updateAutosave(self):
         '''Update autosave timer'''
@@ -568,6 +622,61 @@ class MindMapController(QObject):
         else:
             self.__autosaveTimer.stop()
 
+    def _getSaveFileName(self, title, fileTypes):
+        '''
+        This method call save dialog and return file name for saving
+        @param title: title of the save dialog
+        @type title: QString
+        @param fileTypes: file types for save dialog filter
+        @type fileTypes: hash
+        @return: name of selected file
+        @rtype: str
+        '''
+        filter = ''
+        for type in fileTypes:
+            filter = filter + type[0] + ';;'
+        selectedFilter = fileTypes[0][0]
+        fileName, currentFilter = gui_factory.invokeSaveFileDialog(title, self.fileDirectory(),
+                filter, selectedFilter)
+        if len(currentFilter) == 0:
+            currentFilter = selectedFilter
+        if len(fileName) == 0:
+            return None
+        for type in fileTypes:
+            if type[0] == currentFilter:
+                fileType = type
+                if not re.match(fileType[1], fileName): fileName += fileType[2]
+                break
+        return unicode(fileName)
+
+    def onExportToImage(self):
+        '''Export mind map to image'''
+        fileTypes = [[self.tr('PNG image (*.png)'), '.*\.png', '.png'],
+                [self.tr('JPEG image (*.jpg *.jpeg)'), '.*\.(jpg|jpeg)', '.jpg'],
+                [self.tr('Windows BMP image (*.bmp)'), '.*\.bmp', '.bmp']]
+        fileName = self._getSaveFileName(self.tr('Save as image'), fileTypes)
+        if not fileName:
+            return
+        size = self.__mindMapScene.itemsBoundingRect().size()
+        pixmap = QPixmap(size.width(), size.height())
+        pixmap.fill(Qt.white)
+        painter = QPainter(pixmap)
+        painter.setRenderHint(QPainter.Antialiasing)
+        self.__mindMapScene.render(painter, QRectF(),
+                self.__mindMapScene.itemsBoundingRect())
+        painter.end()
+        pixmap.save(fileName)
+
+    def onPrint(self):
+        '''Print mind map'''
+        printer = QPrinter(QPrinter.HighResolution)
+        printDialog = QPrintDialog(printer, gui_factory.defaultParent())
+        if printDialog.exec_():
+            painter = QPainter(printer)
+            self.__mindMapScene.render(painter, QRectF(),
+                self.__mindMapScene.itemsBoundingRect())
+            painter.end()
+
     def onFileNew(self):
         '''Process file new action'''
         if not self.prohibitChangeMapAction():
@@ -579,9 +688,9 @@ class MindMapController(QObject):
         if not self.prohibitChangeMapAction():
             return
         fileName = QFileDialog.getOpenFileName(gui_factory.defaultParent(), self.tr('Open'),
-                self._fileDirectory(), self.tr('Mind maps (*.mm);;All Files (*)'))[0]
+                self.fileDirectory(), self.tr('Mind maps (*.mm);;All Files (*)'))
         if not fileName: return
-        self.loadFile(fileName)
+        self.loadFile(unicode(fileName))
 
     def onFileSave(self):
         '''
@@ -608,16 +717,15 @@ class MindMapController(QObject):
         @return: True if mind map was saved False otherwise
         @rtype: bool
         '''
-        fileName = QFileDialog.getSaveFileName(gui_factory.defaultParent(),
-                self.tr('Save as'), self._fileDirectory(),
-                self.tr('Mind maps (*.mm);;All Files (*)'))[0]
+        fileName = self._getSaveFileName(self.tr('Save as'),
+                [[self.tr('Mind maps (*.mm)'), '.*\.mm', '.mm']])
         if not fileName: return False
-        if not re.match('.*\.mm', fileName): fileName += '.mm'
         try:
             parser.storeMindMap(fileName, self.__mindMapScene.mindMap)
             self._setFileName(fileName)
             self.__undoStack.setClean()
             self.__autosaved = False
+            settings.addRecentFile(fileName)
             return True
         except IOError, args:
             QMessageBox.critical(gui_factory.defaultParent(), self.tr('File save error'),
@@ -639,12 +747,12 @@ class MindMapController(QObject):
         @param command: command to execute
         @type command: str
         '''
-        if isinstance(command, QUndoCommand):
+        if not isinstance(command, HivemindCommand):
             self.__undoStack.push(command)
-        elif command == 'undo':
-            self.__undoStack.undo()
-        elif command == 'redo':
-            self.__undoStack.redo()
+        elif isinstance(command, ActionCommand):
+            {'undo': self.__undoStack.undo, 'redo': self.__undoStack.redo}[command.action]()
+        elif isinstance(command, CurrentNodeCommand):
+            self.__mindMapScene.currentNode = command.node
 
     def _undoStackStateChanged(self, clean):
         '''
@@ -676,14 +784,18 @@ class MindMapController(QObject):
         @rtype: bool
         '''
         if self.__undoStack.isClean() and not self.__autosaved:
-            self._removeBackup()
+            self._removeAutosave()
             return True
         decition = self._showSaveChangesBox()
         if decition == QMessageBox.Save:
-            return self.onFileSave()
+            if self.onFileSave():
+                self._removeAutosave()
+                return True
+            else:
+                return False
         elif decition == QMessageBox.Cancel:
             return False
-        self._removeBackup()
+        self._removeAutosave()
         return True
 
     def _setFileName(self, fileName):
@@ -694,7 +806,7 @@ class MindMapController(QObject):
         else:
             gui_factory.defaultParent().setWindowTitle(None)
 
-    def _fileDirectory(self):
+    def fileDirectory(self):
         '''
         @return: Directory of the oppenned file or home
         @rtype: str
@@ -702,9 +814,10 @@ class MindMapController(QObject):
         if not self.__fileName: return os.path.expanduser('~')
         return os.path.dirname(self.__fileName)
 
-    def _getBackupFilename(self, path):
+    @classmethod
+    def _generateAutosaveFilename(cls, path):
         '''
-        Generate backup file name for custom file name
+        Generate autosave file name for custom file name
         @type path: unicode
         '''
         if path is not None:
@@ -712,6 +825,16 @@ class MindMapController(QObject):
             return os.path.join(dirname, '.#' + filename + '#')
         return None
 
+    def openRecentFile(self):
+        '''
+        Open file selected from list of recent files
+        '''
+        if not self.prohibitChangeMapAction():
+            return
+        action = QAction.sender(self)
+        if action:
+            self.loadFile(unicode(action.data().toString()))
+
     def loadFile(self, path):
         '''
         Load mind map file and show chosen map.
@@ -719,19 +842,21 @@ class MindMapController(QObject):
         @type path: unicode
         '''
         filePath = path
-        backup = self._getBackupFilename(path)
-        if self._showRecoverDataDialog(backup):
+        if self._showRecoverDataDialog(filePath):
             self.__autosaved = True
-            filePath = backup
+            filePath = self._generateAutosaveFilename(path)
         else:
             self.__autosaved = False
         try:
             mindMap = parser.loadMindMap(filePath) if filePath is not None else MindMap()
-            self._setFileName(filePath)
-            self._loadMindMap(mindMap)
         except Exception:
             QMessageBox.critical(gui_factory.defaultParent(), self.tr('File open error'),
-                    self.tr('Cannot read mind map from file %s') % os.path.split(path)[1])
+                    unicode(self.tr('Cannot read mind map from file %s')) %
+                    os.path.split(path)[1])
+            return
+        self._setFileName(path)
+        self._loadMindMap(mindMap)
+        settings.addRecentFile(path)
 
     def loadMindMapFromNetwork(self, mindMap):
         '''
@@ -757,18 +882,19 @@ class MindMapController(QObject):
         Show dialog to the user asking him/her whether there is a need to recover
         autosaved data copy or not
         
-        @param filename: name of the backup file
+        @param filename: name of the mind map file
         @type filename: str
         
         @return: True if user want to recover data, False otherwise
         @rtype: bool
         '''
         if filename is None: return False
-        if os.path.isfile(filename):
+        autosavedFilename = self._generateAutosaveFilename(filename)
+        if os.path.isfile(autosavedFilename):
             if QMessageBox.question(gui_factory.defaultParent(),
                     self.tr('Autosaved data recovery'),
-                    self.tr('File %s has automatically saved data. Recover it?') %
-                            os.path.split(filename)[1][:-1],
+                    unicode(self.tr('File %s has automatically saved data. Recover it?')) %
+                            os.path.split(filename)[1],
                     QMessageBox.Yes | QMessageBox.No) == QMessageBox.Yes:
                 return True
         return False
@@ -813,7 +939,6 @@ def _getLocale():
 
 def run():
     '''Run HiveMind application'''
-    app = QApplication.instance()
     # translator installation
     locale = _getLocale()[0:2]
     translatorStd = QTranslator()
@@ -823,12 +948,5 @@ def run():
     translator = QTranslator()
     if translator.load(sysconfig.TRANSLATION_PATH_TEMPLATE % locale):
         app.installTranslator(translator)
-    # launch of reactor and controller
-    global reactor
-    reactor.runReturn()
-    # stopping twisted event shuts down application
-    reactor.addSystemEventTrigger('after', 'shutdown', app.quit)
-    # exit from twisted when last window is closed
-    app.connect(app, SIGNAL('lastWindowClosed()'), reactor.stop)
     MainWindowController()
     sys.exit(app.exec_())
