@@ -25,14 +25,15 @@ Dialogs for Maemo platform
 '''
 
 import hivemind.resource as resource
-from PySide.QtGui import QFrame, QWidget, QGridLayout, QStackedWidget, QVBoxLayout, \
+from PyQt4.QtGui import QFrame, QWidget, QGridLayout, QStackedWidget, QVBoxLayout, \
 QRadioButton, QHBoxLayout, QToolBar, QIcon, QPalette, QPushButton, QLabel, \
-QTextBrowser, QAbstractSlider, QTabWidget, QDialog, QFont, QDesktopServices
-from PySide.QtCore import Qt, SIGNAL, QSize, QTimer, QUrl
+QTextBrowser, QTabWidget, QDialog, QFont, QDesktopServices, QScrollArea, \
+QAction, QPainter, QBrush, QPen, QColor
+from PyQt4.QtCore import Qt, SIGNAL, QSize, QTimer, QUrl, QEvent, QObject
 from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
-AboutDialog, EditEdgeDialog, SettingsDialog, AffiliationDialog, NetworkServiceDialog, \
+AboutDialog, EditEdgeDialog, SettingsDialog, AffiliationDialog, NetworkSettingsDialog, \
 NetworkClientDialog, NetworkConnectionDialog
-from hivemind.gui_widgets import ScalingWidget, SettingsCreator, ActionImageButton
+from hivemind.gui_widgets import ScalingWidget, SettingsCreator, ActionImageButton, FindToolBar
 from hivemind.attribute import adaptCallable
 import hivemind.__version__ as __version__
 
@@ -109,8 +110,8 @@ class MaemoEditNodeDialog(MaemoDialogMixin, EditNodeDialog):
         mainLayout.addLayout(bottomLayout)
         self._editor.setFocus()
         self.setLayout(mainLayout)
-        self.__showHtml.connect(SIGNAL('toggled(bool)'), adaptCallable(self._changedMode))
-        self._textHtmlAction.connect(SIGNAL('toggled(bool)'),
+        QObject.connect(self.__showHtml, SIGNAL('toggled(bool)'), adaptCallable(self._changedMode))
+        QObject.connect(self._textHtmlAction, SIGNAL('toggled(bool)'),
                                      adaptCallable(self._showTextSourceButtons))
 
     def _createToolbar(self):
@@ -133,6 +134,8 @@ class MaemoEditNodeDialog(MaemoDialogMixin, EditNodeDialog):
         toolbar.addAction(self._backgroundAction)
         self._clearFormatAction.setIcon(QIcon(resource.getImage('node_editor_clear')))
         toolbar.addAction(self._clearFormatAction)
+        self._editHyperlinkAction.setIcon(QIcon(resource.getImage('node_editor_link')))
+        toolbar.addAction(self._editHyperlinkAction)
         return toolbar
 
     def _changedMode(self, isSourceMode):
@@ -262,30 +265,24 @@ class MaemoAboutDialog(AboutDialog):
         self._reportBug = QPushButton(self.tr('Report bug'))
         self._name = QLabel('<h2>%s %s</h2>' % (self._appName, __version__.VERSION))
         self._about = QTextBrowser()
-        self._about.setHtml(('<br><br><br><br><br><br><br><br><br>' +
-                '<center>%s<br>%s</center>' +
+        self._about.setHtml(('<center>%s</center>' +
                 '<ul style="list-style-type: square;margin: 0px 0px 0px 40px">' +
-                '<li>%s<li>%s<li>%s<li>%s<li>%s</ul><br>' +
-                '<center>%s<br>%s<br><br>%s<br>%s' +
-                '<br><br><br><br><br><br><br><br><br></center>') % (
-                self._license, self._copyright,
+                '<li>%s<li>%s<li>%s<li>%s<li>%s<li>%s</ul><center>%s<br><br>' +
+                '%s<br>%s<br><br>%s<br>%s</center>') % (
+                self._copyright,
                 self._generateAuthorInfo('AndreyVasilev'),
+                self._generateAuthorInfo('IlyaParamonov'),
                 self._generateAuthorInfo('OlegKandaurov'),
                 self._generateAuthorInfo('AlexanderKulikov'),
-                self._generateAuthorInfo('IlyaParamonov'),
+                self._generateAuthorInfo('EldarMamedov'),
                 self._generateAuthorInfo('AndreyGolovchenko'),
+                self._license,
                 self._development, (self._linkTemplate % (self._fructLink, self._fruct)),
                 self._generatePartcipantInfo('NicuBuculei'),
                 self._generatePartcipantInfo('Mentalrey')))
-        self._about.setVerticalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
-        self._about.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
         self._about.setOpenExternalLinks(True)
-        self._scrollBar = self._about.verticalScrollBar()
-        self._scrollBar.setSingleStep(1)
-        self._scrollBar.setRepeatAction(QAbstractSlider.SliderSingleStepAdd, 0, 70)
-        self._scrollBar.connect(SIGNAL('actionTriggered(int)'), adaptCallable(self._scroll))
-        self._visitWebsite.connect(SIGNAL('clicked()'), self._openWebsite)
-        self._reportBug.connect(SIGNAL('clicked()'), self._openBugReport)
+        QObject.connect(self._visitWebsite, SIGNAL('clicked()'), self._openWebsite)
+        QObject.connect(self._reportBug, SIGNAL('clicked()'), self._openBugReport)
 
     def _generatePartcipantInfo(self, name):
         '''
@@ -326,17 +323,6 @@ class MaemoAboutDialog(AboutDialog):
         '''Open bug report page'''
         QDesktopServices.openUrl(QUrl(self._bugLink))
 
-    def _scroll(self, action):
-        '''
-        Handle action from vertical scroll bar
-        @type action: int
-        '''
-        if action == QAbstractSlider.SliderSingleStepAdd:
-            position = self._scrollBar.sliderPosition()
-            self._scrollBar.setSliderPosition(position + self._scrollBar.singleStep())
-            if position == self._scrollBar.sliderPosition():
-                self._scrollBar.triggerAction(QAbstractSlider.SliderToMinimum)
-
 
 class MaemoEditEdgeDialog(MaemoDialogMixin, EditEdgeDialog):
     '''
@@ -382,39 +368,42 @@ class MaemoSettingsDialog(MaemoDialogMixin, SettingsDialog):
         SettingsDialog.__init__(self, parent)
         MaemoDialogMixin.__init__(self)
         tabWidget = QTabWidget()
-        self._general = SettingsCreator(['locale', 'confirmDelete', 'useExtendedMenu'])
+        self._general = SettingsCreator(['locale', 'confirmDelete', 'useExtendedMenu',
+                'autosaveEnabled', 'autosaveInterval'])
         generalTab = QWidget()
         generalTab.setLayout(self._general.getLayout())
+        generalScrollArea = QScrollArea()
+        generalScrollArea.setWidget(generalTab)
+        generalScrollArea.setWidgetResizable(True)
         self._node = SettingsCreator(['defaultFont',
                 'defaultNodeTextColor',
                 'defaultNodeBackgroundColor',
                 'defaultSelectedNodeBackgroundColor'])
         nodeTab = QWidget()
         nodeTab.setLayout(self._node.getLayout())
-        self._edge = SettingsCreator(['defaultEdgeWidth', 'defaultEdgeColor'])
+        self._edge = SettingsCreator(['defaultEdgeWidth', 'defaultEdgeColor',
+                'defaultLabelFont', 'defaultLabelColor'])
         edgeTab = QWidget()
         edgeTab.setLayout(self._edge.getLayout())
-        self._edgeLabel = SettingsCreator(['defaultLabelFont', 'defaultLabelColor'])
-        edgeLabelTab = QWidget()
-        edgeLabelTab.setLayout(self._edgeLabel.getLayout())
-        self._animation = SettingsCreator(['autoHideNodeMenuTimeout',
-                'showNodeMenuAnimationDuration',
-                'hideNodeMenuAnimationDuration'])
-        animationTab = QWidget()
-        animationTab.setLayout(self._animation.getLayout())
-        self._autosave = SettingsCreator(['autosaveEnabled', 'autosaveInterval'])
-        autosaveTab = QWidget()
-        autosaveTab.setLayout(self._autosave.getLayout())
-        self._maxTextWidth = SettingsCreator(['defaultZoomLevel', 'maxNodeTextWidth'])
-        otherTab = QWidget()
-        otherTab.setLayout(self._maxTextWidth.getLayout())
-        tabWidget.addTab(generalTab, self.tr('General'))
+        self._view = SettingsCreator(['autoHideNodeMenuTimeout',
+                'showNodeMenuAnimationDuration', 'hideNodeMenuAnimationDuration',
+                'defaultZoomLevel', 'maxNodeTextWidth'])
+        viewTab = QWidget()
+        viewTab.setLayout(self._view.getLayout())
+        shortcutsTab = QWidget()
+        shortcutsScrollArea = QScrollArea()
+        shortcutsScrollArea.setWidgetResizable(True)
+        self._shortcuts = SettingsCreator(['shortcuts'])
+        shortcutsTab.setLayout(self._shortcuts.getLayout())
+        shortcutsScrollArea.setWidget(shortcutsTab)
+        viewScrollArea = QScrollArea()
+        viewScrollArea.setWidget(viewTab)
+        viewScrollArea.setWidgetResizable(True)
+        tabWidget.addTab(generalScrollArea, self.tr('General'))
         tabWidget.addTab(nodeTab, self.tr('Node'))
-        tabWidget.addTab(edgeTab, self.tr('Edge'))
-        tabWidget.addTab(edgeLabelTab, self.tr('Edge Label'))
-        tabWidget.addTab(animationTab, self.tr('Animation'))
-        tabWidget.addTab(autosaveTab, self.tr('Autosave'))
-        tabWidget.addTab(otherTab, self.tr('View'))
+        tabWidget.addTab(edgeTab, self.tr('Edge && Label'))
+        tabWidget.addTab(viewScrollArea, self.tr('View'))
+        tabWidget.addTab(shortcutsScrollArea, self.tr('Shortcuts'))
         mainLayout = QVBoxLayout()
         mainLayout.addWidget(tabWidget)
         mainLayout.addWidget(self._buttonBox)
@@ -427,10 +416,8 @@ class MaemoSettingsDialog(MaemoDialogMixin, SettingsDialog):
         self._newSettings.append(self._general.retrieve())
         self._newSettings.append(self._node.retrieve())
         self._newSettings.append(self._edge.retrieve())
-        self._newSettings.append(self._edgeLabel.retrieve())
-        self._newSettings.append(self._animation.retrieve())
-        self._newSettings.append(self._autosave.retrieve())
-        self._newSettings.append(self._maxTextWidth.retrieve())
+        self._newSettings.append(self._view.retrieve())
+        self._newSettings.append(self._shortcuts.retrieve())
         SettingsDialog._accept(self)
 
 
@@ -476,13 +463,12 @@ class MaemoExtendedMenuDialog(QDialog):
         def createLabel(text):
             '''Creates label with special properties'''
             label = QLabel(text)
-            label.setWordWrap(True)
             label.setAlignment(Qt.AlignCenter)
             return label
 
         layout = QGridLayout()
         font = QFont()
-        font.setPixelSize(28)
+        font.setPixelSize(24)
         if self.__showHints:
             singleClickLabel = QLabel(self.tr('Single-click on the node to show ' +
                     'the main menu:'))
@@ -490,39 +476,50 @@ class MaemoExtendedMenuDialog(QDialog):
             doubleClickLabel = QLabel(self.tr('Double-click on the node to show the ' +
                     'additional menu:'))
             doubleClickLabel.setFont(font)
-            layout.addWidget(singleClickLabel, 0, 0, 1, 4, Qt.AlignCenter)
-            layout.addWidget(doubleClickLabel, 3, 0, 1, 4, Qt.AlignCenter)
-        layout.addWidget(self._createActionWidget(actionBag.addNodeAction), 1, 0,
+            layout.addWidget(singleClickLabel, 0, 0, 1, 12, Qt.AlignCenter)
+            layout.addWidget(doubleClickLabel, 3, 0, 1, 12, Qt.AlignCenter)
+        layout.addWidget(self._createActionWidget(actionBag.addNodeAction), 1, 0, 1, 3,
                 Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Add node')), 2, 0, Qt.AlignHCenter | Qt.AlignTop)
-        layout.addWidget(self._createActionWidget(actionBag.editNodeAction), 1, 1,
+        layout.addWidget(createLabel(self.tr('Add node')), 2, 0, 1, 3,
+                Qt.AlignHCenter | Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.editNodeAction), 1, 3, 1, 3,
                 Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Edit node')), 2, 1, Qt.AlignHCenter |
+        layout.addWidget(createLabel(self.tr('Edit node')), 2, 3, 1, 3,
+                Qt.AlignHCenter | Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.removeNodeAction), 1, 6, 1, 3,
+                Qt.AlignCenter)
+        layout.addWidget(createLabel(self.tr('Remove node')), 2, 6, 1, 3,
+                Qt.AlignHCenter | Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.foldNodeAction), 1, 9, 1, 3,
+                Qt.AlignCenter)
+        if actionBag.foldNodeAction.isChecked():
+            foldLabel = createLabel(self.tr('Unfold node subtree'))
+        else:
+            foldLabel = createLabel(self.tr('Fold node subtree'))
+        foldLabel.setWordWrap(True)
+        layout.addWidget(foldLabel, 2, 9, 1, 3, Qt.AlignHCenter | Qt.AlignTop)
+
+        layout.addWidget(self._createActionWidget(actionBag.editNodeIconsAction),
+                6, 0, 1, 4, Qt.AlignCenter)
+        layout.addWidget(createLabel(self.tr('Select icons')), 7, 0, 1, 4,
+                Qt.AlignHCenter | Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.editEdgeAction), 6, 4, 1, 4,
+                Qt.AlignCenter)
+        layout.addWidget(createLabel(self.tr('Edit edge')), 7, 4, 1, 4, Qt.AlignHCenter |
                 Qt.AlignTop)
-        layout.addWidget(self._createActionWidget(actionBag.removeNodeAction), 1, 2,
-                Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Remove node')), 2, 2, Qt.AlignHCenter |
+        layout.addWidget(self._createActionWidget(actionBag.enterTransferModeAction),
+                6, 8, 1, 4, Qt.AlignCenter)
+        layout.addWidget(createLabel(self.tr('Move node')), 7, 8, 1, 4, Qt.AlignHCenter |
                 Qt.AlignTop)
-        layout.addWidget(self._createActionWidget(actionBag.foldNodeAction), 1, 3,
+
+        layout.addWidget(self._createActionWidget(actionBag.editLabelAction), 4, 2, 1, 4,
                 Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Fold/unfold node subtree')),
-                2, 3, Qt.AlignHCenter | Qt.AlignTop)
-        layout.addWidget(self._createActionWidget(actionBag.editNodeIconsAction), 4, 0,
-                Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Select icons')), 5, 0, Qt.AlignHCenter |
-                Qt.AlignTop)
-        layout.addWidget(self._createActionWidget(actionBag.editEdgeAction), 4, 1,
-                Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Edit edge')), 5, 1, Qt.AlignHCenter |
-                Qt.AlignTop)
-        layout.addWidget(self._createActionWidget(actionBag.enterTransferModeAction), 4, 2,
-                Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Move node')), 5, 2, Qt.AlignHCenter |
-                Qt.AlignTop)
-        layout.addWidget(self._createActionWidget(actionBag.editLabelAction), 4, 3,
-                Qt.AlignCenter)
-        layout.addWidget(createLabel(self.tr('Edit edge label')), 5, 3, Qt.AlignHCenter |
-                Qt.AlignTop)
+        layout.addWidget(createLabel(self.tr('Edit edge label')), 5, 2, 1, 4,
+                Qt.AlignHCenter | Qt.AlignTop)
+        layout.addWidget(self._createActionWidget(actionBag.openHyperlinkAction),
+                4, 6, 1, 4, Qt.AlignCenter)
+        layout.addWidget(createLabel(self.tr('Go to hyperlink')), 5, 6, 1, 4,
+                Qt.AlignHCenter | Qt.AlignTop)
         self.setLayout(layout)
 
     def _unfreeze(self):
@@ -539,18 +536,18 @@ class MaemoExtendedMenuDialog(QDialog):
         @type action: QAction
         '''
         button = ActionImageButton(action)
-        button.connect(SIGNAL('clicked()'), self.accept)
+        QObject.connect(button, SIGNAL('clicked()'), self.accept)
         return button
 
 
-class MaemoNetworkServiceDialog(MaemoDialogMixin, NetworkServiceDialog):
+class MaemoNetworkSettingsDialog(MaemoDialogMixin, NetworkSettingsDialog):
     '''
-    Service dialog for Maemo platfrom
+    Network settings dialog for Maemo platfrom
     @author: Oleg Kandaurov
     '''
 
     def __init__(self, parent = None):
-        NetworkServiceDialog.__init__(self, parent)
+        NetworkSettingsDialog.__init__(self, parent)
         MaemoDialogMixin.__init__(self)
 
 
@@ -574,3 +571,178 @@ class MaemoNetworkConnectionDialog(MaemoDialogMixin, NetworkConnectionDialog):
     def __init__(self, parent = None):
         NetworkConnectionDialog.__init__(self, parent)
         MaemoDialogMixin.__init__(self)
+
+
+class MaemoNetworkMainDialog(MaemoDialogMixin, QDialog):
+    '''
+    Main network dialog for Maemo platform
+    @author: Oleg Kandaurov
+    '''
+    def __init__(self, actionBag, parent = None):
+        '''Constructor'''
+        QDialog.__init__(self, parent)
+        MaemoDialogMixin.__init__(self)
+        self.__bag = actionBag
+        self._createComponents()
+        self._initComponents()
+        self._connectActions()
+
+    def _createComponents(self):
+        '''Create components'''
+        self.__editAffiliationButton = QPushButton(self.__bag.editAffiliationAction.text())
+        self.__disconnectXMPPButton = QPushButton(self.__bag.disconnectXMPPAction.text())
+        self.__stopProtocolButton = QPushButton(self.__bag.stopProtocolAction.text())
+        self.__startServiceButton = QPushButton(self.__bag.startServiceAction.text())
+        self.__startClientButton = QPushButton(self.__bag.startClientAction.text())
+        self.__networkSettingsButton = QPushButton(self.__bag.networkSettingsAction.text())
+        self.__networkHelpButton = QPushButton(self.__bag.networkHelpAction.text())
+
+    def _initComponents(self):
+        '''Init components'''
+        mainLayout = QVBoxLayout()
+        mainLayout.addWidget(self.__startServiceButton)
+        mainLayout.addWidget(self.__startClientButton)
+        mainLayout.addWidget(self.__stopProtocolButton)
+        mainLayout.addWidget(self.__disconnectXMPPButton)
+        mainLayout.addWidget(self.__editAffiliationButton)
+        layout = QHBoxLayout()
+        layout.addWidget(self.__networkHelpButton)
+        layout.addWidget(self.__networkSettingsButton)
+        mainLayout.addLayout(layout)
+        self.setLayout(mainLayout)
+
+    def _connectActions(self):
+        '''Connect actions'''
+        self.__editAffiliationButton.clicked.connect(self.__bag.editAffiliationAction.trigger)
+        self.__disconnectXMPPButton.clicked.connect(self.__bag.disconnectXMPPAction.trigger)
+        self.__stopProtocolButton.clicked.connect(self.__bag.stopProtocolAction.trigger)
+        self.__startServiceButton.clicked.connect(self.__bag.startServiceAction.trigger)
+        self.__startClientButton.clicked.connect(self.__bag.startClientAction.trigger)
+        self.__networkHelpButton.clicked.connect(self.__bag.networkHelpAction.trigger)
+        self.__networkSettingsButton.clicked.connect(self.__bag.networkSettingsAction.trigger)
+        self.__bag.editAffiliationAction.changed.connect(self._updateState)
+        self.__bag.disconnectXMPPAction.changed.connect(self._updateState)
+        self.__bag.stopProtocolAction.changed.connect(self._updateState)
+        self.__bag.startServiceAction.changed.connect(self._updateState)
+        self.__bag.startClientAction.changed.connect(self._updateState)
+        self.__bag.networkHelpAction.changed.connect(self._updateState)
+        self.__bag.networkSettingsAction.changed.connect(self._updateState)
+
+    def _updateState(self):
+        '''Update button states acccording to action state'''
+        self.__editAffiliationButton.setEnabled(self.__bag.editAffiliationAction.isEnabled())
+        self.__disconnectXMPPButton.setEnabled(self.__bag.disconnectXMPPAction.isEnabled())
+        self.__stopProtocolButton.setEnabled(self.__bag.stopProtocolAction.isEnabled())
+        self.__startServiceButton.setEnabled(self.__bag.startServiceAction.isEnabled())
+        self.__startClientButton.setEnabled(self.__bag.startClientAction.isEnabled())
+        self.__networkHelpButton.setEnabled(self.__bag.networkHelpAction.isEnabled())
+        self.__networkSettingsButton.setEnabled(self.__bag.networkSettingsAction.isEnabled())
+
+
+class MaemoFindToolBar(FindToolBar):
+    '''
+    Find tool bar for Desktop platform
+    '''
+
+    __HEIGHT = 82
+    '''Height of the tool bar'''
+
+    __WIDTH = 630
+    '''Width of the tool bar'''
+
+    __RADIUS = 20
+    '''Corner radius'''
+
+    def __init__(self, controller, parent = None):
+        '''
+        @type controller: MainWindowController
+        @type parent: QWidget
+        '''
+        FindToolBar.__init__(self, controller, parent)
+        self._findWidget = QWidget()
+        self.resize(self.__WIDTH, self.__HEIGHT)
+        self.setAllowedAreas(Qt.NoToolBarArea)
+        self.setMovable(False)
+        self.setFloatable(False)
+        self._createComponents()
+        self._initComponents()
+        parent.installEventFilter(self)
+
+    def _createComponents(self):
+        '''
+        Create components
+        '''
+        #pylint: disable = W0201
+        FindToolBar._createComponents(self)
+        self.__prevAction = QAction(self)
+        self.__nextAction = QAction(self)
+        self.__closeAction = QAction(self)
+        self.__prevAction.setIcon(QIcon(resource.getImage('find_previous')))
+        self.__nextAction.setIcon(QIcon(resource.getImage('find_next')))
+        self.__closeAction.setIcon(QIcon(resource.getImage('find_close')))
+        QObject.connect(self.__nextAction, SIGNAL('triggered()'), self._finder.nextFoundNode)
+        QObject.connect(self.__prevAction, SIGNAL('triggered()'), self._finder.prevFoundNode)
+        QObject.connect(self.__closeAction, SIGNAL('triggered()'), self.close)
+        self.__prevButton = ActionImageButton(self.__prevAction)
+        self.__nextButton = ActionImageButton(self.__nextAction)
+        self.__closeButton = ActionImageButton(self.__closeAction)
+
+    def _initComponents(self):
+        '''
+        Init components
+        '''
+        buttonLayout = QHBoxLayout()
+        buttonLayout.setSpacing(30)
+        buttonLayout.addWidget(self.__prevButton)
+        buttonLayout.addWidget(self.__nextButton)
+        mainLayout = QHBoxLayout()
+        mainLayout.addWidget(self._findLabel)
+        mainLayout.addWidget(self._findLineEdit)
+        mainLayout.addLayout(buttonLayout)
+        mainLayout.addStretch()
+        mainLayout.addWidget(self.__closeButton)
+        self._findWidget.setLayout(mainLayout)
+        self.addWidget(self._findWidget)
+        self._findLineEdit.setStyleSheet("QLineEdit { border-width: 8px;"
+                 + " border-color: white; border-style: solid;"
+                 + " border-radius: 12px; background: white; }")
+
+    def paintEvent(self, paintEvent):
+        '''
+        Paint black background
+        @param paintEvent: paint event representation
+        @type paintEvent: QPaintEvent
+        '''
+        painter = QPainter(self)
+        background = QColor(self.backgroundRole())
+        background.setAlpha(128)
+        brush = QBrush(background)
+        pen = QPen(background)
+        painter.fillRect(0, self.__RADIUS, self.__WIDTH, self.__HEIGHT - self.__RADIUS, brush)
+        painter.fillRect(self.__RADIUS, 0, self.__WIDTH - self.__RADIUS, self.__RADIUS, brush)
+        painter.setPen(pen)
+        painter.setBrush(brush)
+        painter.drawPie(0, 0, (self.__RADIUS - 1) * 2, (self.__RADIUS - 1) * 2, 180 * 8,
+                180 * 8)
+
+    def eventFilter(self, obj, event):
+        '''
+        Catch Windows state changed and Resize events from parent object
+        @type obj: QEvent
+        @type event: QEvent
+        '''
+        parent = self.parentWidget()
+        if obj != self.parent():
+            return QToolBar.eventFilter(self, obj, event)
+        isFullScreen = parent.windowState() & Qt.WindowFullScreen
+        if event.type() == QEvent.Resize:
+            if isFullScreen:
+                self.resize(self.width() - parent.fullScreenExitButton.width(), self.__HEIGHT)
+                self.move(parent.width() - self.width() - parent.fullScreenExitButton.width(),
+                         parent.height() - parent.fullScreenExitButton.height())
+            else:
+                self.resize(self.__WIDTH, self.__HEIGHT)
+                self.move(parent.width() - self.width() + 10, parent.height() - self.height()
+                        + 10)
+        return QToolBar.eventFilter(self, obj, event)
+
