@@ -21,7 +21,5 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02111-1301  USA
 
 '''
-***Comment here***
-
-@author: 
+Network subsystem
 '''
