@@ -23,12 +23,13 @@
 '''
 Network subsystem of the application
 '''
+from twisted.names.srvconnect import SRVConnector
 
 from idavoll.backend import PubSubServiceFromBackend, BackendService
 from idavoll.memory_storage import Storage, LeafNode
 from wokkel.xmppim import RosterClientProtocol
 from wokkel.pubsub import PubSubClient, Subscription, NS_PUBSUB_ERRORS, NS_PUBSUB_EVENT
-from wokkel.client import HybridAuthenticator, XMPPClientConnector
+from wokkel.client import HybridAuthenticator
 from wokkel.ping import PingHandler, NS_PING
 from wokkel.generic import stripNamespace
 from wokkel.subprotocols import StreamManager, XMPPHandler, XMPPHandlerCollection
@@ -37,17 +38,19 @@ from twisted.words.xish.xmlstream import STREAM_ERROR_EVENT
 from twisted.words.xish.domish import Element
 from twisted import application
 from twisted.internet.task import LoopingCall
+from twisted.internet import reactor
 from twisted import internet
 from twisted.python import log
+from hivemind.commands import StateModificationCommand, CurrentNodeCommand
 from hivemind.attribute import Observable, writable, readable
 from hivemind.network.network_models import AffiliationModel, RosterModel
 from hivemind.network import error_messages
-from hivemind.gui_main import reactor
 from hivemind import gui_factory, parser
 import hivemind, uuid, copy, idavoll
-from PySide.QtCore import QObject
+from PyQt4.QtCore import QObject
 #pylint: disable=C0103
 
+
 ''' XMPP PubSub node name '''
 XMPP_NODE_NAME = unicode('HiveMind')
 ''' Time in seconds we must wait for unsubscribe '''
@@ -87,13 +90,13 @@ class NetworkController(QObject):
         self.__affiliationDialog = None
         self.__networkDialog = None
         self.__clientDialog = None
-        self.__serviceDialog = None
-        self.__afterNetworkDialog = None
+        self.__networkSettingsDialog = None
+        self.__secondConnectionStep = None
         self.__errorDialog = None
         self.__pingManager = None
         self._initDialogs()
         self._connectActions()
-        self._initActions(transport = False, protocol = False)
+        self._updateActions('disconnected')
 
     def _initDialogs(self):
         '''initialize network dialogs'''
@@ -101,74 +104,79 @@ class NetworkController(QObject):
             '''Process accepted signal from affiliation dialog'''
             self.__storage.rootNode.affiliations = self.__affiliationModel.affiliations
 
-        def networkAccepted():
-            '''Process accepted signal from network dialog'''
+        def startNetwork():
+            '''Start network connection'''
             self._startTransport(self.__networkDialog.attributes['userJid'] + '/hivemind',
-                    self.__networkDialog.attributes['password'])
-            if self.__afterNetworkDialog:
-                self.__afterNetworkDialog.exec_()
-            self.__afterNetworkDialog = None
-
-        def clientAccepted():
-            '''Process accepted signal from client dialog'''
-            self._configureClient(self.__clientDialog.attributes['serviceJid'] + '/hivemind')
-            self._startProtocol()
-
-        def serviceAccepted():
-            '''Process accepted signal from service dialog'''
-            self._configureService(self.__serviceDialog.attributes['readOnly'],
-                    self.__serviceDialog.attributes['accessModel'])
-            self._startProtocol()
+                    self.__networkDialog.attributes['password'],
+                    self.__networkDialog.attributes['host'],
+                    self.__networkDialog.attributes['port'])
+            reactor.callLater(2, self._performSecondConnectionStep)
 
         self.__networkDialog = gui_factory.createNetworkConnectionDialog()
-        self.__networkDialog.accepted.connect(networkAccepted)
+        self.__networkDialog.accepted.connect(startNetwork)
         self.__clientDialog = gui_factory.createNetworkClientDialog(self.__rosterModel)
-        self.__clientDialog.accepted.connect(clientAccepted)
-        self.__serviceDialog = gui_factory.createNetworkServiceDialog()
-        self.__serviceDialog.accepted.connect(serviceAccepted)
+        self.__clientDialog.accepted.connect(self._startClient)
         self.__affiliationDialog = gui_factory.createAffiliationDialog(self.__affiliationModel)
         self.__affiliationDialog.accepted.connect(affiliationAccepted)
         self.__errorDialog = gui_factory.createNetworkErrorDialog()
+        self.__networkSettingsDialog = gui_factory.createNetworkSettingsDialog()
+
+    def _startClient(self):
+        '''Start client'''
+        self._configureClient(self.__clientDialog.attributes['serviceJid'] + '/hivemind')
+        self._startProtocol()
+
+    def _startService(self):
+        '''Start service'''
+        self._configureService()
+        self._startProtocol()
+
+    def _performSecondConnectionStep(self):
+        ''' Perform second connection step'''
+        if self.__secondConnectionStep and self.__xmppClient is not None:
+            {'client': self.__clientDialog.exec_,
+                        'service': self._startService}[self.__secondConnectionStep]()
+        self.__secondConnectionStep = None
 
     def _connectActions(self):
         '''Connect actions'''
-        def showStartServiceDialog():
-            '''Show service dialog after network dialog'''
-            self.__afterNetworkDialog = self.__serviceDialog
-            showNetworkDialog()
+        def prepareService():
+            '''Second connection step is service'''
+            self.__secondConnectionStep = 'service'
 
-        def showStartClientDialog():
-            '''Show client dialog after network dialog'''
-            self.__afterNetworkDialog = self.__clientDialog
-            showNetworkDialog()
+        def prepareClient():
+            '''Second connection step is client'''
+            self.__secondConnectionStep = 'client'
 
         def showNetworkDialog():
-            '''Show appropriate network dialog'''
+            '''Perform actions according to network state'''
             if self.__actionBag.disconnectXMPPAction.isEnabled():
-                self.__afterNetworkDialog.exec_()
+                self._performSecondConnectionStep()
             else:
                 self.__networkDialog.exec_()
 
         self.__actionBag.disconnectXMPPAction.triggered.connect(self._closeNetwork)
         self.__actionBag.stopProtocolAction.triggered.connect(self._closeProtocolLevel)
-        self.__actionBag.startServiceAction.triggered.connect(showStartServiceDialog)
-        self.__actionBag.startClientAction.triggered.connect(showStartClientDialog)
+        self.__actionBag.startServiceAction.triggered.connect(prepareService)
+        self.__actionBag.startServiceAction.triggered.connect(showNetworkDialog)
+        self.__actionBag.startClientAction.triggered.connect(prepareClient)
+        self.__actionBag.startClientAction.triggered.connect(showNetworkDialog)
         self.__actionBag.editAffiliationAction.triggered.connect(
                 self.__affiliationDialog.exec_)
+        self.__actionBag.networkSettingsAction.triggered.connect(
+                self.__networkSettingsDialog.exec_)
 
-    def _initActions(self, transport, protocol):
+    def _updateActions(self, state):
         '''
-        Initialize actions
-        @param transport: Whether transport connected or not
-        @type transport: bool
-        @param protocol: Whether protocol connected or not
-        @type protocol: bool
+        Update actions according to state
+        @type state: str
         '''
-        self.__actionBag.disconnectXMPPAction.setEnabled(transport)
-        self.__actionBag.stopProtocolAction.setEnabled(transport and protocol)
-        self.__actionBag.startServiceAction.setDisabled(transport and protocol)
-        self.__actionBag.startClientAction.setDisabled(transport and protocol)
-        self.__actionBag.editAffiliationAction.setEnabled(transport and protocol and
+        self.__actionBag.connectionStatusAction.status = state
+        self.__actionBag.disconnectXMPPAction.setEnabled(state != 'disconnected')
+        self.__actionBag.stopProtocolAction.setEnabled(state == 'collaboration')
+        self.__actionBag.startServiceAction.setDisabled(state == 'collaboration')
+        self.__actionBag.startClientAction.setDisabled(state == 'collaboration')
+        self.__actionBag.editAffiliationAction.setEnabled(state == 'collaboration' and
                 self.__isServiceRole)
 
     def publishCommand(self, command):
@@ -176,43 +184,52 @@ class NetworkController(QObject):
         Publish command in common space
         @param command: command to publish in shared space
         '''
-        data = parser.serializeCommand(command)
-        if self.__protocol:
-            self.__protocol.proposeChangeset(Changeset(data, 'command'))
+        if self.__protocol and self._isPropagationAllowed(command):
+            self.__protocol.proposeChangeset(Changeset(parser.serializeCommand(command),
+                    'command'))
         else:
             self.__mindMapController.executeCommand(command)
 
-    def _startTransport(self, userJid, password):
+    def _isPropagationAllowed(self, command):
+        '''
+        Check if command must be propagated through network or not
+        @param command: command to inspect
+        '''
+        if hivemind.settings.get('presentationMode'):
+            return True
+        if isinstance(command, StateModificationCommand):
+            if 'folded' in command.stateAfter.keys():
+                return False
+        if isinstance(command, CurrentNodeCommand):
+            return False
+        return True
+
+    def _startTransport(self, userJid, password, host, port):
         '''
         Start XMPP connection
         @param userJid: JID for XMPP connection
         @type userJid: str
         @param password: password for XMPP connection
         @type password: str
+        @type host: str
+        @type port: int
         '''
         authenticator = HybridAuthenticator(jabber.jid.internJID(userJid), password)
         factory = HivemindXmlStreamFactory(authenticator)
         factory.addObserver(self)
-        self.__xmppClient = HivemindXMPPClient(authenticator, factory)
+        self.__xmppClient = HivemindXMPPClient(authenticator, factory, host, port)
         self.__xmppClient.addObserver(self)
         self.__xmppClient.logTraffic = True
         self.__xmppClient.startService()
 
-    def _configureService(self, readOnly, accessModel):
-        '''
-        Configure HiveMind service
-        @param readOnly: specifies whether mind map read only or not
-        @type readOnly: bool
-        @param accessModel: Access model of service
-        @type accessModel: str
-        '''
+    def _configureService(self):
+        '''Configure HiveMind service'''
         self.__isServiceRole = True
         self.__storage = HivemindStorage(self, self.__rosterModel)
         self.__protocol = HivemindService(HivemindBackendService(self.__storage))
         mindMap = parser.mindMapToString(self.__mindMapController.initialMindMap, False)
         undoStack = parser.serializeCommandStack(self.__mindMapController.undoStack)
-        config = self.__storage.generateConfig(accessModel, readOnly)
-        self.__storage.createRootNode(self.__xmppClient.jid, config,
+        self.__storage.createRootNode(self.__xmppClient.jid,
                 Changeset(mindMap, 'mindmap'), Changeset(undoStack, 'undostack'))
 
     def _configureClient(self, service):
@@ -286,6 +303,7 @@ class NetworkController(QObject):
         if self.__xmppClient:
             self.__xmppClient.stopService()
             self.__xmppClient = None
+        self._updateActions('disconnected')
 
     def _closeProtocolLevel(self):
         '''Close protocol level'''
@@ -315,10 +333,8 @@ class NetworkController(QObject):
 
     def processNotification(self, *values, **kvargs):
         '''Process notification'''
-        if 'transport' in kvargs:
-            self._processTransportState(kvargs['transport'])
-        elif 'protocol' in kvargs:
-            self._processProtocolState(kvargs['protocol'])
+        if 'connection' in kvargs:
+            self._processConnectionState(kvargs['connection'])
         elif 'subscription' in kvargs:
             self._processSubscriptionEvent(kvargs['subscription'])
         elif 'error' in kvargs:
@@ -335,10 +351,8 @@ class NetworkController(QObject):
 
     def _processTransportPingEvent(self, event):
         '''Process ping event from transport'''
-        if event['state'] == 'alive':
-            self._updateActions('alive')
-        elif event['state'] == 'timeout':
-            self._updateActions('disconnected')
+        if event['state'] == 'timeout':
+            self._processConnectionState('disconnected')
             if self.__pingManager:
                 self.__pingManager.stopPing(event['entity'])
             #Do tricky reconnect
@@ -365,7 +379,7 @@ class NetworkController(QObject):
     def _processRosterEvent(self, event):
         '''Process event from roster'''
         if event['type'] == 'roster':
-            self.__rosterModel.roster = event['data']
+            self.__rosterModel.setRoster(event['data'])
         elif event['type'] == 'remove':
             self.__rosterModel.removeContact(event['data'])
         elif event['type'] == 'set':
@@ -434,18 +448,6 @@ class NetworkController(QObject):
             else:
                 self._handleStanzaError(failure)
 
-    def _processProtocolState(self, state):
-        '''
-        Process protocol state
-        @type state: str
-        '''
-        mapState = {'on': 'yes', 'off': 'no'}
-        if state == 'on':
-            self._startProtocolHelpers()
-        self.__actionBag.subscribeStatusAction.status = mapState[state]
-        self._initActions(self.__actionBag.connectionStatusAction.status != 'no',
-            self.__actionBag.subscribeStatusAction.status != 'no')
-
     def _processSubscriptionEvent(self, event):
         '''
         Process subscription event
@@ -464,29 +466,18 @@ class NetworkController(QObject):
             if self.__pingManager:
                 self.__pingManager.removePinger(event['entity'])
 
-    def _processTransportState(self, state):
+    def _processConnectionState(self, state):
         '''
-        Process transport state
-        @param state: Current transport state
+        Process connection state
         @type state: str
         '''
-        self._updateActions(state)
-        if state == 'initialized':
+        if state == 'collaboration':
+            self._startProtocolHelpers()
+        elif state == 'initialized':
             self._startTransportHelpers()
         elif state == 'disconnected':
             self._closeNetwork()
-
-    def _updateActions(self, state):
-        '''
-        Update actions according to transport state
-        @type state: str
-        '''
-        mapState = {'initialized': 'established', 'alive': 'established',
-                'connecting': 'connecting', 'disconnected': 'no',
-                'connected': self.__actionBag.connectionStatusAction.status}
-        self.__actionBag.connectionStatusAction.status = mapState[state]
-        self._initActions(self.__actionBag.connectionStatusAction.status != 'no',
-                self.__actionBag.subscribeStatusAction.status != 'no')
+        self._updateActions(state)
 
     def _executeChangeset(self, changeset):
         '''
@@ -521,7 +512,7 @@ class HivemindXmlStreamFactory(jabber.xmlstream.XmlStreamFactory, Observable):
         '''
         log.msg('Connection started at %s' % self.__class__.__name__)
         jabber.xmlstream.XmlStreamFactory.startedConnecting(self, connector)
-        self.notifyObservers(transport = 'connecting')
+        self.notifyObservers(connection = 'connecting')
 
     def clientConnectionFailed(self, connector, reason):
         '''
@@ -727,8 +718,9 @@ class HivemindXMPPClient(HivemindStreamManager, application.service.Service, Obs
     @author: Oleg Kandaurov
     '''
 
+    #pylint: disable=W0212
     readable('server')
-    def __init__(self, authenticator, factory, host = None, port = 5222):
+    def __init__(self, authenticator, factory, host = None, port = None):
         '''
         @type authenticator: HybridAuthenticator
         @type factory: XmlStreamFactory
@@ -739,7 +731,7 @@ class HivemindXMPPClient(HivemindStreamManager, application.service.Service, Obs
         self.domain = authenticator.jid.host
         self.__server = jabber.jid.internJID(self.domain)
         self.host = host
-        self.port = port
+        self.port = hivemind.settings.get('defaultXMPPPort') if port is None else port
         self._connection = None
         HivemindStreamManager.__init__(self, factory)
         Observable.__init__(self)
@@ -774,7 +766,7 @@ class HivemindXMPPClient(HivemindStreamManager, application.service.Service, Obs
         self.jid = self.factory.authenticator.jid
         StreamManager._authd(self, xs)
         log.msg('Connection initialized at %s' % (self.__class__.__name__))
-        self.notifyObservers(transport = 'initialized')
+        self.notifyObservers(connection = 'initialized')
 
     def _connected(self, xs):
         '''Called when the transport connection has been established'''
@@ -783,18 +775,18 @@ class HivemindXMPPClient(HivemindStreamManager, application.service.Service, Obs
             _notifyError(error, self)
         StreamManager._connected(self, xs)
         log.msg('Transport connected at %s' % (self.__class__.__name__))
-        self.notifyObservers(transport = 'connected')
+        self.notifyObservers(connection = 'connected')
         xs.addObserver(STREAM_ERROR_EVENT, _onError)
 
     def _disconnected(self, reason):
         '''Called when the stream has been closed'''
         StreamManager._disconnected(self, reason)
         log.msg('Stream closed at %s' % (self.__class__.__name__))
-        self.notifyObservers(transport = 'disconnected')
+        self.notifyObservers(connection = 'disconnected')
 
     def _createConnector(self):
         '''Create connector instance'''
-        connector = XMPPClientConnector(reactor, self.domain, self.factory)
+        connector = HivemindXMPPClientConnector(reactor, self.domain, self.port, self.factory)
         connector.connect()
         return connector
 
@@ -805,6 +797,25 @@ class HivemindXMPPClient(HivemindStreamManager, application.service.Service, Obs
     connection = property(_getConnection, None, None, 'Client connector')
 
 
+class HivemindXMPPClientConnector(SRVConnector):
+    '''
+    Hivemind implementation of XMPP connector
+    @author: Oleg Kandaurov
+    '''
+    def __init__(self, reactor, domain, port, factory):
+        '''Constructor'''
+        SRVConnector.__init__(self, reactor, 'xmpp-client', domain, factory)
+        self.__port = port
+
+    def pickServer(self):
+        host, port = SRVConnector.pickServer(self)
+        if not self.servers and not self.orderedServers:
+            port = 5222
+        if self.__port is not None:
+            port = self.__port
+        return host, port
+
+
 class HivemindRosterClientProtocol(RosterClientProtocol, Observable):
     '''
     Client side roster protocol for Hivemind
@@ -861,12 +872,13 @@ class HivemindService(PubSubServiceFromBackend, Observable):
 
     def connectionInitialized(self):
         '''The XML stream has been initialized'''
+        #pylint: disable=W0201
         self.serviceJID = self.parent.jid
-        self.notifyObservers(protocol = 'on')
+        self.notifyObservers(connection = 'collaboration')
 
     def close(self):
         '''Close service'''
-        self.notifyObservers(protocol = 'off')
+        self.notifyObservers(connection = 'initialized')
         return internet.defer.succeed(None)
 
     def subscribe(self, requestor, service, nodeIdentifier, subscriber):
@@ -933,7 +945,7 @@ class HivemindClient(PubSubClient, Observable):
         self.__jid = self.parent.jid
         defer = self.subscribe(self.__service, XMPP_NODE_NAME, self.__jid)
         defer.addCallback(self.retrieveItems)
-        defer.addCallback(lambda success: self.notifyObservers(protocol = 'on'))
+        defer.addCallback(lambda success: self.notifyObservers(connection = 'collaboration'))
         defer.addErrback(_notifyError, self)
 
     def close(self):
@@ -948,7 +960,7 @@ class HivemindClient(PubSubClient, Observable):
             log.msg('Unsubscribed from %s' % self.__service.full())
             if unsubscribeTimer.active():
                 unsubscribeTimer.cancel()
-            self.notifyObservers(protocol = 'off')
+            self.notifyObservers(connection = 'initialized')
             return None
 
         unsubscribeTimer = reactor.callLater(UNSUBSCRIBE_TIMEOUT, notifyUnsubscription, self)
@@ -1040,7 +1052,8 @@ class HivemindBackendService(BackendService):
             '''Check affiliation for node'''
             if affiliation == 'outcast':
                 raise idavoll.error.Forbidden()
-            elif affiliation == 'member':
+            if (node.getConfiguration()['pubsub#publish_model'] == 'publishers' and
+                    affiliation == 'member'):
                 raise idavoll.error.ItemForbidden()
             return node
         defer = node.getAffiliation(requestor)
@@ -1051,7 +1064,8 @@ class HivemindBackendService(BackendService):
         '''Publish particular items to the node'''
         def checkSubscription(node):
             '''Check requestor subscription for the current node'''
-            if not node.isSubscribed(requestor):
+            if (not node.isSubscribed(requestor) and
+                    node.getConfiguration()['pubsub#publish_model'] != 'all'):
                 raise idavoll.error.NotSubscribed()
             return node
         defer = self.storage.getNode(nodeIdentifier)
@@ -1077,33 +1091,21 @@ class HivemindStorage(Storage):
         self.__rosterModel = rosterModel
         self.__rootNode = None
 
-    def generateConfig(self, accessModel, readOnly):
-        '''
-        Generate configuration file for leaf node
-        @type accessModel: str
-        @type readOnly: str
-        '''
-        config = copy.copy(Storage.defaultConfig['leaf'])
-        config['pubsub#node_type'] = 'leaf'
-        config['pubsub#send_last_published_item'] = 'never'
-        config['pubsub#access_model'] = accessModel
-        config['pubsub#publish_model'] = 'subscribers'
-        if readOnly:
-            config['pubsub#publish_model'] = 'publishers'
-        return config
-
-    def createRootNode(self, owner, config, mindMap, undoStack):
+    def createRootNode(self, owner, mindMap, undoStack):
         '''
         Creates root node of the storage
         @param owner: Owner of the node
         @type owner: JID
-        @param config: Node configuration
-        @type config: dict
         @param mindMap: initial record in the service with whole mind map
         @type mindMap: Changeset
         @param undoStack: undostack associated with mindMap
         @type undoStack: Changeset
         '''
+        config = copy.copy(self.defaultConfig['leaf'])
+        config['pubsub#node_type'] = 'leaf'
+        config['pubsub#send_last_published_item'] = 'never'
+        config['pubsub#access_model'] = hivemind.settings.get('accessModel')
+        config['pubsub#publish_model'] = hivemind.settings.get('publishModel')
         node = HivemindNode(self, XMPP_NODE_NAME, owner, config)
         node.addObserver(self.__parent)
         self._nodes[XMPP_NODE_NAME] = node
@@ -1137,6 +1139,12 @@ class HivemindNode(LeafNode, Observable, object):
         self.__stack = ChangesetStack()
         self.__owner = owner
 
+    def getConfiguration(self):
+        '''Get configuration according to network settings'''
+        self._config['pubsub#access_model'] = hivemind.settings.get('accessModel')
+        self._config['pubsub#publish_model'] = hivemind.settings.get('publishModel')
+        return self._config
+
     def storeItems(self, items, publisher):
         '''
         Store items in persistent storage for later retrieval
@@ -1231,10 +1239,7 @@ class HivemindNode(LeafNode, Observable, object):
         '''
         if entity.userhost() in self._affiliations:
             return
-        affiliation = 'publisher'
-        if self._config['pubsub#publish_model'] == 'publishers':
-            affiliation = 'member'
-        self.setAffiliation(entity, affiliation)
+        self.setAffiliation(entity, hivemind.settings.get('defaultAffiliation'))
 
     def getAffiliation(self, entity):
         '''
@@ -1350,7 +1355,7 @@ class Changeset(object):
         '''
         xmlChangeset = element.firstChildElement()
         stripNamespace(xmlChangeset)
-        changeset = xmlChangeset.toXml()
+        changeset = xmlChangeset.toXml().encode('utf8')
         return Changeset(changeset, element.getAttribute('type'),
                 jabber.jid.JID(element.getAttribute('publisher')), element.getAttribute('id'),
                 element.getAttribute('number'))
@@ -1580,3 +1585,11 @@ def _pubsubConditionFromStanzaError(error):
         return None, None
     return error.appCondition.name, error.appCondition.getAttribute('feature')
 
+
+def startReactor():
+    '''Starts the reactor and defines exit behaviour'''
+    reactor.runReturn()
+    # stopping twisted event shuts down application
+    reactor.addSystemEventTrigger('after', 'shutdown', reactor.qApp.quit)
+    # stop twisted event loop when last window is closed
+    reactor.qApp.lastWindowClosed.connect(reactor.stop)
