@@ -23,8 +23,8 @@
 '''
 Network related models and delegates
 '''
-from PySide.QtGui import QStyledItemDelegate, QComboBox
-from PySide.QtCore import Qt, QAbstractListModel, QModelIndex, QAbstractTableModel
+from PyQt4.QtGui import QStyledItemDelegate, QComboBox
+from PyQt4.QtCore import Qt, QAbstractListModel, QModelIndex, QAbstractTableModel
 from hivemind.attribute import readable
 
 class RosterModel(QAbstractListModel):
@@ -125,7 +125,7 @@ class AffiliationModel(QAbstractTableModel):
         self.__mapStates = {'subscribed': self.tr('Connected'),
                 'unsubscribed': self.tr('Disconnected')}
         self.__mapAffiliations = {'publisher': self.tr('Full'),
-                'member' : self.tr('Read only'),
+                'member' : self.tr('Restricted'),
                 'outcast': self.tr('None'),
                 'owner': self.tr('Owner')}
         self.__column = {'jid': 0, 'affiliation': 1, 'state': 2}
