@@ -29,11 +29,15 @@ Application resource manager
 import os
 import hivemind.settings as settings
 import hivemind.sysconfig as sysconfig
-from PySide.QtGui import QIcon, QImage, QPixmap, QPixmapCache
+from PyQt4.QtGui import QIcon, QImage, QPixmap, QPixmapCache
+from PyQt4.QtSvg import QSvgRenderer
 
 __ICONS = {}
 '''Hash table containing icons'''
 
+__SVG_ICONS = {}
+'''Hash table containing svg icons'''
+
 def getIconByName(name):
     '''
     Get the image with the given name.
@@ -45,6 +49,17 @@ def getIconByName(name):
         return __ICONS[name]
     return None
 
+def getSvgIconByName(name):
+    '''
+    Get the svg image with the given name.
+    If the name is not present, supply None instead
+
+    @rtype: QImage
+    '''
+    if __SVG_ICONS.has_key(name):
+        return __SVG_ICONS[name]
+    return None
+
 def getImage(name):
     '''
     @param name: name of image file without extension
@@ -52,7 +67,26 @@ def getImage(name):
     @return: new image object on first call, cached image afterwards 
     @rtype: QPixmap
     '''
-    fullname = os.path.join(sysconfig.BASEDIR, 'images', name + '.png')
+    return _getImage(name, '.png')
+
+def getScalableImage(name):
+    '''
+    @param name: name of image file without extension
+
+    @return: new image object on first call, cached image afterwards
+    @rtype: QPixmap
+    '''
+    return _getImage(name, '.svg')
+
+def _getImage(name, extension):
+    '''
+    @param name: name of image file without extension
+    @param extension: image file extension
+
+    @return: new image object on first call, cached image afterwards
+    @rtype: QPixmap
+    '''
+    fullname = os.path.join(sysconfig.BASEDIR, 'images', name + extension)
     image = QPixmap()
     if not QPixmapCache.find(fullname, image):
         image = QPixmap(fullname)
@@ -93,6 +127,11 @@ def _loadIcons():
         image = QImage(fullPath)
         if image:
             __ICONS[name] = image
+    for name in settings.get("listSvgIcons"):
+        fullPath = os.path.join(path, name + '.svg')
+        image = QSvgRenderer(fullPath)
+        if image:
+            __SVG_ICONS[name] = image
 
 # Initialization
 _loadIcons()
