@@ -27,25 +27,20 @@ Application settings manager
 @author: Ilya Paramonov
 '''
 
-from PySide.QtCore import QSettings, QObject, SIGNAL
-from PySide.QtGui import QFont, QColor
+from PyQt4.QtCore import QSettings, QObject, SIGNAL, QFileInfo
+from PyQt4.QtGui import QFont, QColor
 import os
 import hivemind.sysconfig as sysconfig
 
 __INT_PARAMETERS = ('defaultEdgeWidth', 'autoHideNodeMenuTimeout',
         'showNodeMenuAnimationDuration', 'hideNodeMenuAnimationDuration',
         'transportPingInterval', 'transportPingMaxCount', 'protocolPingInterval',
-        'protocolPingMaxCount', 'autosaveInterval',
-        'defaultZoomLevel', 'maxNodeTextWidth')
+        'protocolPingMaxCount', 'autosaveInterval', 'maxRecentFiles',
+        'defaultZoomLevel', 'maxNodeTextWidth', 'defaultXMPPPort', 'sidebarAnimationDuration')
 '''
 Integer parameters names
 '''
 
-__DOUBLE_PARAMETERS = ()
-'''
-Float parameters names
-'''
-
 __FONT_PARAMETERS = ('defaultFont', 'defaultLabelFont')
 '''
 Font parameters names
@@ -59,7 +54,7 @@ Font parameters names
 '''
 
 __BOOLEAN_PARAMETERS = ('confirmDelete', 'autosaveEnabled', 'readOnly', 'firstClick',
-                        'useExtendedMenu')
+                        'useExtendedMenu', 'presentationMode')
 '''
 Boolean parameters names
 '''
@@ -69,10 +64,21 @@ __HIDDEN_PARAMETERS = ('password')
 Hidden parameters name
 '''
 
+__LIST_PARAMETERS = ('listIcons', 'listSvgIcons', 'recentFiles')
+'''
+List parameters name
+'''
+
+__CACHED_ATTRIBUTES = {}
+'''
+Cached attributes
+'''
+
 def translationDir():
     '''Return directory where application translations reside'''
     return "%s/translations" % sysconfig.BASEDIR
 
+
 def get(parameter):
     '''
     Get parameter by name
@@ -80,25 +86,33 @@ def get(parameter):
     @return: parameter value
     @rtype: according to parameter type
     '''
+    if parameter in __CACHED_ATTRIBUTES:
+        return __CACHED_ATTRIBUTES[parameter]
     value = __settings.value(parameter)
     if value is None and (parameter not in __HIDDEN_PARAMETERS):
         raise Exception('Invalid parameter %s' % parameter)
     if parameter in __INT_PARAMETERS:
-        return int(value)
-    elif parameter in __DOUBLE_PARAMETERS:
-        return value
+        __CACHED_ATTRIBUTES[parameter] = value.toInt()[0]
     elif parameter in __FONT_PARAMETERS:
         font = QFont()
-        font.fromString(value)
-        return font
+        font.fromString(value.toString())
+        __CACHED_ATTRIBUTES[parameter] = font
+    elif parameter in __LIST_PARAMETERS:
+        parameterList = unicode(value.toString()).split(',')
+        if '' in parameterList:
+            parameterList.remove('')
+        __CACHED_ATTRIBUTES[parameter] = parameterList
     elif parameter in __COLOR_PARAMETERS:
-        return QColor(value)
+        __CACHED_ATTRIBUTES[parameter] = QColor(value.toString())
     elif parameter in __BOOLEAN_PARAMETERS:
-        if value is True or value is False:
-            return value
-        return {'true': True, 'false': False}.get(value.lower())
+        return value.toBool()
+#        if value is True or value is False:
+#            return value
+#        __CACHED_ATTRIBUTES[parameter] = {'true': True, 'false': False}.get(value.lower())
     else:
-        return value
+        __CACHED_ATTRIBUTES[parameter] = unicode(value.toString())
+    return __CACHED_ATTRIBUTES[parameter]
+
 
 def set(**parameters): #pylint: disable=W0622
     '''
@@ -109,18 +123,37 @@ def set(**parameters): #pylint: disable=W0622
             variant = paramValue.toString()
         elif paramName in __COLOR_PARAMETERS:
             variant = paramValue.name()
+        elif paramName in __LIST_PARAMETERS:
+            listStr = ','.join(paramValue)
+            variant = listStr
         else:
             variant = paramValue
+        __CACHED_ATTRIBUTES[paramName] = paramValue
         __settings.setValue(paramName, variant)
         __signalizer.emit(SIGNAL('settingsChanged'))
 
+def addRecentFile(fileName):
+    '''
+    Add fileName to the list of recent files
+    @param fileName: name of the recent edited file
+    @type fileName: str
+    '''
+    #pylint: disable=E1103
+    recentFiles = get('recentFiles')
+    if fileName is not None:
+        fileName = unicode(QFileInfo(fileName).absoluteFilePath())
+        if fileName in recentFiles:
+            recentFiles.remove(fileName)
+        recentFiles.insert(0, fileName)
+        set(recentFiles = recentFiles[:get('maxRecentFiles')])
+
 def connect(receiver):
     '''
     Subscribe receiver to be called when settings changes
     
     @param receiver: callable object
     '''
-    __signalizer.connect(SIGNAL('settingsChanged'), receiver)
+    QObject.connect(__signalizer, SIGNAL('settingsChanged'), receiver)
 
 def _reinitialize(path):
     '''
@@ -131,6 +164,7 @@ def _reinitialize(path):
     
     @param path: path to the user scope configuration file
     '''
+    __CACHED_ATTRIBUTES.clear()
     QSettings.setPath(QSettings.NativeFormat, QSettings.UserScope, path)
     global __settings
     __settings = QSettings(QSettings.NativeFormat, QSettings.UserScope,
